@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,36 +31,31 @@ void htaccess(const char *name)
 {
    char htname[MAXLEN];
    FILE *fp_auth;
+   int i;
 
-   if(strncmp(UserAuthentication,"yes",3) !=0 )
+   if(!UserAuthentication)
       return;
 
    sprintf(htname,"%s/%s/.htaccess",dirname,name);
    if((fp_auth=fopen(htname,"w"))==NULL) {
-      fprintf(stderr, "SARG: (auth) %s: %s\n",text[45],htname);
+      fprintf(stderr, "SARG: (auth) %s: %s - %s\n",text[45],htname,strerror(errno));
       exit(1);
    }
 
-   strcpy(warea,Require);
-   if (getword(buf,sizeof(buf),warea,'%')<0) {
-      printf("SARG: The \"Require\" entry of your sarg.conf file is too long for your %s file.\n",htname);
-      exit(1);
+   fprintf(fp_auth,"AuthUserFile %s\n",AuthUserFile);
+   if(strchr(AuthName,'\"') == 0)
+      fprintf(fp_auth,"AuthName \"%s\"\n",AuthName);
+   else
+      fprintf(fp_auth,"AuthName %s\n",AuthName);
+   fprintf(fp_auth,"AuthType %s\n",AuthType);
+   fputs("<Limit GET POST>\nRequire ",fp_auth);
+   for (i=0 ; Require[i] ; i++)
+      if (Require[i]=='%' && Require[i+1]=='u') {
+         fputs(name,fp_auth);
+         i++;
+      } else {
+         fputc(Require[i],fp_auth);
    }
-   fputs("AuthUserFile ",fp_auth);
-   fputs(AuthUserFile,fp_auth);
-   fputs("\n",fp_auth);
-   fputs("AuthName ",fp_auth);
-   if(strstr(AuthName,"\"") == 0)
-      fputs("\"",fp_auth);
-   fputs(AuthName,fp_auth);
-   if(strstr(AuthName,"\"") == 0)
-      fputs("\"",fp_auth);
-   fputs("\n",fp_auth);
-   fputs("AuthType ",fp_auth);
-   fputs(AuthType,fp_auth);
-   fputs("\n<Limit GET POST>\n",fp_auth);
-   fputs(buf,fp_auth);
-   fputs(name,fp_auth);
    fputs("\n</LIMIT>\n",fp_auth);
 
    fclose(fp_auth);
