@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,8 +31,8 @@ void authfail_report(void)
 {
 
    FILE *fp_in = NULL, *fp_ou = NULL;
-      
-   char url[MAXLEN];
+
+   char url[2*MAXLEN];
    char authfail_in[MAXLEN];
    char per[MAXLEN];
    char report[MAXLEN];
@@ -47,13 +48,14 @@ void authfail_report(void)
    int  z=0;
    int  count=0;
    int  cstatus;
+   struct getwordstruct gwarea;
 
-   if(strlen(DataFile) > 0) return;
+   if(DataFile[0] != '\0') return;
 
    ouser[0]='\0';
 
    sprintf(tmp4,"%s/sarg/authfail.log.unsort",TempDir);
-   
+
    if(!authfail_count) {
       unlink(tmp4);
       return;
@@ -117,9 +119,10 @@ void authfail_report(void)
    fprintf(fp_ou,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91]);
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if (getword_multisep(data,sizeof(data),buf,' ')<0 || getword_multisep(hora,sizeof(hora),buf,' ')<0 ||
-          getword_multisep(user,sizeof(user),buf,' ')<0 || getword_multisep(ip,sizeof(ip),buf,' ')<0 ||
-          getword_multisep(url,sizeof(url),buf,' ')<0) {
+      getword_start(&gwarea,buf);
+      if (getword(data,sizeof(data),&gwarea,'\t')<0 || getword(hora,sizeof(hora),&gwarea,'\t')<0 ||
+          getword(user,sizeof(user),&gwarea,'\t')<0 || getword(ip,sizeof(ip),&gwarea,'\t')<0 ||
+          getword(url,sizeof(url),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",authfail_in);
          exit(1);
       }
@@ -129,7 +132,7 @@ void authfail_report(void)
             fixip(user);
       }
 
-      if(strcmp(Ip2Name,"yes") == 0) 
+      if(Ip2Name)
          ip2name(ip,sizeof(ip));
 
       if(!z) {
@@ -141,7 +144,7 @@ void authfail_report(void)
             user[0]='\0';
          if(user[0] != '\0')
             strcpy(ouser,user);
-         if(strcmp(oip,ip) == 0) 
+         if(strcmp(oip,ip) == 0)
             ip[0]='\0';
          if(ip[0] != '\0')
             strcpy(oip,ip);
