@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,8 +31,8 @@ void gen_denied_report(void)
 {
 
    FILE *fp_in = NULL, *fp_ou = NULL;
-      
-   char url[MAXLEN];
+
+   char url[2*MAXLEN];
    char denied_in[MAXLEN];
    char per[MAXLEN];
    char report[MAXLEN];
@@ -46,6 +47,7 @@ void gen_denied_report(void)
    char *str;
    int  z=0;
    int  count=0;
+   struct getwordstruct gwarea;
 
    ouser[0]='\0';
    ouser2[0]='\0';
@@ -103,9 +105,10 @@ void gen_denied_report(void)
    fprintf(fp_ou,"<tr><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th><th class=\"header\">%s</th></tr>\n",text[98],text[111],text[110],text[91]);
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if (getword(data,sizeof(data),buf,' ')<0 || getword(hora,sizeof(hora),buf,' ')<0 ||
-          getword(user,sizeof(user),buf,' ')<0 || getword(ip,sizeof(ip),buf,' ')<0 ||
-          getword(url,sizeof(url),buf,' ')<0) {
+      getword_start(&gwarea,buf);
+      if (getword(data,sizeof(data),&gwarea,'\t')<0 || getword(hora,sizeof(hora),&gwarea,'\t')<0 ||
+          getword(user,sizeof(user),&gwarea,'\t')<0 || getword(ip,sizeof(ip),&gwarea,'\t')<0 ||
+          getword(url,sizeof(url),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",denied_in);
          exit(1);
       }
@@ -115,7 +118,7 @@ void gen_denied_report(void)
             fixip(user);
       }
 
-      if(strcmp(Ip2Name,"yes") == 0) 
+      if(Ip2Name)
          ip2name(ip,sizeof(ip));
 
       if(!z) {
@@ -127,7 +130,7 @@ void gen_denied_report(void)
             user[0]='\0';
          if(user[0] != '\0')
             strcpy(ouser,user);
-         if(strcmp(oip,ip) == 0) 
+         if(strcmp(oip,ip) == 0)
             ip[0]='\0';
          if(ip[0] != '\0')
             strcpy(oip,ip);
@@ -141,7 +144,7 @@ void gen_denied_report(void)
 
       if(DeniedReportLimit) {
          if(strcmp(ouser2,name) == 0) {
-            count++; 
+            count++;
          } else {
             count=1;
             strcpy(ouser2,name);
@@ -150,7 +153,7 @@ void gen_denied_report(void)
             continue;
       }
 
-      if(strlen(BlockIt) > 0)
+      if(BlockIt[0] != '\0')
          sprintf(BlockImage,"<a href=\"%s%s?url=%s\"><img src=\"%s/sarg-squidguard-block.png\" border=\"0\"></a>&nbsp;",wwwDocumentRoot,BlockIt,url,ImageFile);
       else BlockImage[0]='\0';
 
