@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -28,6 +29,17 @@
 
 extern numlist hours, weekdays;
 
+static int is_param(const char *param,const char *buf)
+{
+   int plen;
+
+   plen=strlen(param);
+   if (strncmp(buf,param,plen) != 0) return(0);
+   buf+=plen;
+   if ((unsigned char)*buf>' ') return(0);
+   return(1);
+}
+
 static int getparam_string(const char *param,char *buf,char *value,int value_size)
 {
    int plen;
@@ -135,9 +147,34 @@ static int getparam_int(const char *param,char *buf,int *value)
    return(1);
 }
 
+static int getparam_bool(const char *param,char *buf,int *value)
+{
+   int plen;
+   int i;
+   const char *bool_str="yes,true,on,1";
+
+   plen=strlen(param);
+   if (strncmp(buf,param,plen) != 0) return(0);
+   buf+=plen;
+   if ((unsigned char)*buf>' ') return(0);
+   while (*buf && (unsigned char)*buf<=' ') buf++;
+
+   *value=0;
+   for ( ; *bool_str ; bool_str+=i) {
+      for (i=0 ; bool_str[i] && bool_str[i]!=',' ; i++);
+      if (strncasecmp(bool_str,buf,i)==0) {
+         *value=1;
+         break;
+      }
+      if (bool_str[i]==',') i++;
+   }
+   return(1);
+}
+
 static void parmtest(char *buf)
 {
    char wbuf[50];
+   struct getwordstruct gwarea;
 
    while (*buf && (unsigned char)*buf<=' ') buf++;
 
@@ -163,9 +200,9 @@ static void parmtest(char *buf)
 
    if (getparam_string("background_image",buf,BgImage,sizeof(BgImage))>0) return;
 
-   if (getparam_string("show_sarg_info",buf,ShowSargInfo,sizeof(ShowSargInfo))>0) return;
+   if (getparam_bool("show_sarg_info",buf,&ShowSargInfo)>0) return;
 
-   if (getparam_string("show_sarg_logo",buf,ShowSargLogo,sizeof(ShowSargLogo))>0) return;
+   if (getparam_bool("show_sarg_logo",buf,&ShowSargLogo)>0) return;
 
    if (getparam_string("font_face",buf,FontFace,sizeof(FontFace))>0) return;
 
@@ -183,30 +220,31 @@ static void parmtest(char *buf)
 
    if (getparam_quoted("title",buf,Title,sizeof(Title))>0) return;
 
-   if (getparam_string("resolve_ip",buf,Ip2Name,sizeof(Ip2Name))>0) return;
+   if (getparam_bool("resolve_ip",buf,&Ip2Name)>0) return;
 
-   if (getparam_string("user_ip",buf,UserIp,sizeof(UserIp))>0) return;
+   if (getparam_bool("user_ip",buf,&UserIp)>0) return;
 
    if (getparam_string("max_elapsed",buf,MaxElapsed,sizeof(MaxElapsed))>0) return;
 
-   if(strstr(buf,"date_format") != 0) {
-      if (getword_multisep(wbuf,sizeof(wbuf),buf,' ')<0) {
+   if (is_param("date_format",buf)) {
+      getword_start(&gwarea,buf);
+      if (getword_multisep(wbuf,sizeof(wbuf),&gwarea,' ')<0) {
          printf("SARG: Maybe you have a broken record or garbage in date_format parameter.\n");
          exit(1);
       }
-      strncpy(DateFormat,buf,1);
+      strncpy(DateFormat,gwarea.current,1);
       fixnone(DateFormat);
       return;
    }
 
-   if( strstr( buf, "hours" ) != 0 ) {
+   if (is_param("hours",buf)) {
       if( getnumlist( buf, &hours, 24, 24 ) ) {
          fprintf( stderr, "Error: Invalid syntax in hours tag!\n" );
          exit( 1 );
       }
    }
 
-   if( strstr( buf, "weekdays" ) != 0 ) {
+   if (is_param("weekdays",buf)) {
       if( getnumlist( buf, &weekdays, 7, 7 ) ) {
          fprintf( stderr, "Error: Invalid syntax in weekdays tag!\n" );
          exit( 1 );
@@ -217,7 +255,17 @@ static void parmtest(char *buf)
 
    if (getparam_2words("user_sort_field",buf,UserSortField,sizeof(UserSortField),UserSortOrder,sizeof(UserSortOrder))>0) return;
 
-   if (getparam_string("access_log",buf,AccessLog,sizeof(AccessLog))>0) return;
+   if (is_param("access_log",buf)>0) {
+      if (AccessLogFromCmdLine==0) {
+         if (NAccessLog>=MAXLOGS) {
+            fprintf(stderr,"SARG: Too many log files.\n");
+            exit(1);
+         }
+         getparam_string("access_log",buf,AccessLog[NAccessLog],MAXLEN);
+         NAccessLog++;
+      }
+      return;
+   }
 
    if (getparam_string("useragent_log",buf,UserAgentLog,sizeof(UserAgentLog))>0) return;
 
@@ -237,11 +285,14 @@ static void parmtest(char *buf)
 
    if (getparam_string("output_email",buf,OutputEmail,sizeof(OutputEmail))>0) return;
 
-   if (getparam_2words("per_user_limit",buf,PerUserLimitFile,sizeof(PerUserLimitFile),PerUserLimit,sizeof(PerUserLimit))>0) return;
+   if (getparam_2words("per_user_limit",buf,PerUserLimitFile,sizeof(PerUserLimitFile),wbuf,sizeof(wbuf))>0) {
+      PerUserLimit=atoi(wbuf);
+      return;
+   }
 
-   if (getparam_string("lastlog",buf,LastLog,sizeof(LastLog))>0) return;
+   if (getparam_int("lastlog",buf,&LastLog)>0) return;
 
-   if (getparam_string("remove_temp_files",buf,RemoveTempFiles,sizeof(RemoveTempFiles))>0) return;
+   if (getparam_bool("remove_temp_files",buf,&RemoveTempFiles)>0) return;
 
    if (getparam_string("replace_index",buf,ReplaceIndex,sizeof(ReplaceIndex))>0) return;
 
@@ -249,15 +300,15 @@ static void parmtest(char *buf)
 
    if (getparam_string("index",buf,Index,sizeof(Index))>0) return;
 
-   if (getparam_string("overwrite_report",buf,OverwriteReport,sizeof(OverwriteReport))>0) return;
+   if (getparam_bool("overwrite_report",buf,&OverwriteReport)>0) return;
 
    if (getparam_string("records_without_userid",buf,RecordsWithoutUser,sizeof(RecordsWithoutUser))>0) return;
 
-   if (getparam_string("use_comma",buf,UseComma,sizeof(UseComma))>0) return;
+   if (getparam_bool("use_comma",buf,&UseComma)>0) return;
 
    if (getparam_string("mail_utility",buf,MailUtility,sizeof(MailUtility))>0) return;
 
-   if (getparam_string("topsites_num",buf,TopSitesNum,sizeof(TopSitesNum))>0) return;
+   if (getparam_int("topsites_num",buf,&TopSitesNum)>0) return;
 
    if (getparam_int("topuser_num",buf,&TopUsersNum)>0) return;
 
@@ -267,7 +318,7 @@ static void parmtest(char *buf)
 
    if (getparam_2words("topsites_sort_order",buf,TopsitesSortField,sizeof(TopsitesSortField),TopsitesSortType,sizeof(TopsitesSortType))>0) return;
 
-   if (getparam_string("long_url",buf,LongUrl,sizeof(LongUrl))>0) return;
+   if (getparam_bool("long_url",buf,&LongUrl)>0) return;
 
    if (getparam_string("language",buf,language,sizeof(language))>0) return;
 
@@ -292,24 +343,22 @@ static void parmtest(char *buf)
 
    if (getparam_quoted("exclude_string",buf,ExcludeString,sizeof(ExcludeString))>0) return;
 
-   if (getparam_string("privacy",buf,Privacy,sizeof(Privacy))>0) return;
+   if (getparam_bool("privacy",buf,&Privacy)>0) return;
 
-   if (getparam_quoted("privacy_string",buf,ExcludeString,sizeof(ExcludeString))>0) return;
+   if (getparam_quoted("privacy_string",buf,PrivacyString,sizeof(PrivacyString))>0) return;
 
    if (getparam_string("privacy_string_color",buf,PrivacyStringColor,sizeof(PrivacyStringColor))>0) return;
 
-   if (getparam_string("show_successful_message",buf,SuccessfulMsg,sizeof(SuccessfulMsg))>0) return;
+   if (getparam_bool("show_successful_message",buf,&SuccessfulMsg)>0) return;
 
-   if (getparam_string("show_read_statistics",buf,ShowReadStatistics,sizeof(ShowReadStatistics))>0) return;
+   if (getparam_bool("show_read_statistics",buf,&ShowReadStatistics)>0) return;
 
    if (getparam_string("topuser_fields",buf,TopUserFields,sizeof(TopUserFields))>0) return;
 
-   if (getparam_string("bytes_in_sites_users_report",buf,BytesInSitesUsersReport,sizeof(BytesInSitesUsersReport))>0) return;
+   if (getparam_bool("bytes_in_sites_users_report",buf,&BytesInSitesUsersReport)>0) return;
 
    if (getparam_string("user_report_fields",buf,UserReportFields,sizeof(UserReportFields))>0) return;
 
-   if (getparam_string("bytes_in_sites_users_report",buf,BytesInSitesUsersReport,sizeof(BytesInSitesUsersReport))>0) return;
-
    if (getparam_string("datafile",buf,DataFile,sizeof(DataFile))>0) return;
 
    if (getparam_quoted("datafile_delimiter",buf,DataFileDelimiter,sizeof(DataFileDelimiter))>0) return;
@@ -344,7 +393,7 @@ static void parmtest(char *buf)
 
    if (getparam_string("external_css_file",buf,ExternalCSSFile,sizeof(ExternalCSSFile))>0) return;
 
-   if (getparam_string("user_authentication",buf,UserAuthentication,sizeof(UserAuthentication))>0) return;
+   if (getparam_bool("user_authentication",buf,&UserAuthentication)>0) return;
 
    if (getparam_string("AuthUserFile",buf,AuthUserFile,sizeof(AuthUserFile))>0) return;
 
@@ -354,17 +403,23 @@ static void parmtest(char *buf)
 
    if (getparam_string("Require",buf,Require,sizeof(Require))>0) return;
 
-   if (getparam_quoted("download_suffix",buf,DownloadSuffix,sizeof(DownloadSuffix))>0) return;
+   if (is_param("download_suffix",buf)) {
+      char warea[MAXLEN];
 
-   if (getparam_string("graphs",buf,Graphs,sizeof(Graphs))>0) return;
+      getparam_quoted("download_suffix",buf,warea,sizeof(warea));
+      set_download_suffix(warea);
+      return;
+   }
+
+   if (getparam_bool("graphs",buf,&Graphs)>0) return;
 
    if (getparam_string("graph_days_bytes_bar_color",buf,GraphDaysBytesBarColor,sizeof(GraphDaysBytesBarColor))>0) return;
 
    if (getparam_string("squidguard_log_format",buf,SquidGuardLogFormat,sizeof(SquidGuardLogFormat))>0) return;
 
-   if (getparam_string("squidguard_ignore_date",buf,SquidguardIgnoreDate,sizeof(SquidguardIgnoreDate))>0) return;
+   if (getparam_bool("squidguard_ignore_date",buf,&SquidguardIgnoreDate)>0) return;
 
-   if (getparam_string("dansguardian_ignore_date",buf,DansguardianIgnoreDate,sizeof(DansguardianIgnoreDate))>0) return;
+   if (getparam_bool("dansguardian_ignore_date",buf,&DansguardianIgnoreDate)>0) return;
 
    if (getparam_string("ulimit",buf,Ulimit,sizeof(Ulimit))>0) return;
 
@@ -378,22 +433,25 @@ static void parmtest(char *buf)
 
    if (getparam_int("realtime_access_log_lines",buf,&realtime_access_log_lines)>0) return;
 
+   if (getparam_string("graph_font",buf,GraphFont,sizeof(GraphFont))>0) return;
+
    if(strstr(buf,"squid24") != 0) {
       squid24++;
       return;
    }
 
    if(strstr(buf,"byte_cost") != 0) {
-      if (getword_multisep(wbuf,sizeof(wbuf),buf,' ')<0) {
+      getword_start(&gwarea,buf);
+      if (getword_multisep(wbuf,sizeof(wbuf),&gwarea,' ')<0) {
          printf("SARG: Maybe you have a broken record or garbage in byte_cost parameter.\n");
          exit(1);
       }
-      cost=atol(buf);
-      if (getword_multisep(wbuf,sizeof(wbuf),buf,' ')<0) {
+      cost=atol(gwarea.current);
+      if (getword_multisep(wbuf,sizeof(wbuf),&gwarea,' ')<0) {
          printf("SARG: Maybe you have a broken record or garbage in byte_cost parameter.\n");
          exit(1);
       }
-      nocost=my_atoll(buf);
+      nocost=my_atoll(gwarea.current);
       return;
    }
 
