@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -26,6 +27,8 @@
 #include "include/conf.h"
 #include "include/defs.h"
 
+#if defined(HAVE_GD)
+
 static int blue;
 static int white;
 static int lavender;
@@ -36,10 +39,8 @@ static int goldenrod2;
 static int gray;
 static int silver;
 static int black;
-//static char *font1 = FONTDIR"/FreeSans.ttf";
-static char *font1 = FONTDIR"/DejaVuSans.ttf";
 
-#if defined(HAVE_GD) && defined(HAVE_ICONV_H) && defined(gdFTEX_Unicode)
+#if defined(HAVE_ICONV_H) && defined(gdFTEX_Unicode)
 #include <iconv.h>
 #define SARGgdImageStringFT I18NgdImageStringFT
 
@@ -62,8 +63,17 @@ static char * I18NgdImageStringFT (gdImage * im, int *brect, int fg, char *fontl
    sstr = (char *) sstring;
 
    localtoutf = iconv_open ("UTF-8", CharSet);
-   iconv (localtoutf, (ICONV_CONST char **)&str, &slen, &sstr, &sslen);
-   iconv_close (localtoutf);
+   if (localtoutf==(iconv_t)-1) {
+      fprintf(stderr,"SARG(grepday) iconv cannot convert from %s to UTF-8 - %s\n",CharSet,strerror(errno));
+      strcpy(sstring,string); //show something sensible on the graph
+   }
+   else {
+      if (iconv (localtoutf, (ICONV_CONST char **)&str, &slen, &sstr, &sslen)==-1) {
+         fprintf(stderr,"SARG(grepday) iconv failed to convert string \"%s\" from %s to UTF-8 - %s\n",string,CharSet,strerror(errno));
+         strcpy(sstring,string); //show something sensible on the graph
+      }
+      iconv_close (localtoutf);
+   }
 
    retval = gdImageStringFTEx (im, brect, fg, fontlist, ptsize, angle, x, y, sstring, gdFTEX_Unicode);
    free(sstring);
@@ -76,7 +86,6 @@ static char * I18NgdImageStringFT (gdImage * im, int *brect, int fg, char *fontl
 
 static void bar(int x1,long long int n)
 {
-#ifdef HAVE_GD
    gdPoint points[4];
    int brect[8];
    int val=0, x;
@@ -145,11 +154,6 @@ static void bar(int x1,long long int n)
     3900000000LL,4000000000LL,4100000000LL,4200000000LL,4300000000LL,4400000000LL,
     4500000000LL,4600000000LL,4700000000LL,4800000000LL,4900000000LL,5000000000LL};
 
-   if(access(font1, R_OK) != 0) {
-     fprintf(stderr, "SARG: (grepday) Fontname: %s not found.\n",font1);
-     exit(1);
-   }
-
    if(strcmp(GraphDaysBytesBarColor,"orange") == 0) {
       color1 = gdImageColorAllocate(im, 255, 233, 142);
       color2 = gdImageColorAllocate(im, 220, 163, 72);
@@ -214,7 +218,7 @@ static void bar(int x1,long long int n)
 
    snprintf(v,6,"%s",fixnum(num,0));
 
-   SARGgdImageStringFT(im,&brect[0],black,font1,6,0.0,x1-1,val-12,v);
+   SARGgdImageStringFT(im,&brect[0],black,GraphFont,6,0.0,x1-1,val-12,v);
 
    points[0].x = x1+17;
    points[0].y = val-5;
@@ -226,37 +230,45 @@ static void bar(int x1,long long int n)
    points[3].y = 420;
    gdImageFilledPolygon(im, points, 4, color2);
 
-#endif
    return;
 }
+#endif //HAVE_GD
 
 void greport_day(const char *user)
 {
+#ifdef HAVE_GD
    FILE *fp_in, *pngout;
    int x, y;
    int x1;
    int brect[8];
    char wdirname[MAXLEN];
    char graph[MAXLEN];
+   char buf[MAXLEN];
    char wuser[255];
    char csort[255];
    char data[20];
    int  count=0;
-   char oday[20];
-   char day[20];
-   char bytes[20];
+   int oday=0;
+   int day=0;
+   long long int llday;
+   long long int bytes;
+   char ftime[128];
    long long int tot=0;
    time_t t;
    struct tm *local;
    int cstatus;
    char s[15];
+   struct getwordstruct gwarea;
+   struct getwordstruct gwarea1;
 
-#ifdef HAVE_GD
-
-   if(strcmp(Graphs,"yes") != 0) {
+   if(!Graphs || GraphFont[0]=='\0') {
       unlink(wdirname);
       return;
    }
+   if(access(GraphFont, R_OK) != 0) {
+      fprintf(stderr, "SARG: (grepday) Fontname: %s not found.\n",GraphFont);
+      exit(1);
+   }
 
    im = gdImageCreate(720, 480);
 
@@ -315,66 +327,69 @@ void greport_day(const char *user)
    y=65;
    for(x=1; x<=31; x++) {
       sprintf(s,"%02d",x);
-      SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,y,437,s);
+      SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,y,437,s);
       y=y+20;
    }
 
    t = time(NULL);
    local = localtime(&t);
    if(strcmp(DateFormat,"u") == 0)
-      strftime(ftime, 127, "%b/%d/%Y %H:%M", local);
+      strftime(ftime, sizeof(ftime), "%b/%d/%Y %H:%M", local);
    if(strcmp(DateFormat,"e") == 0)
-      strftime(ftime, 127, "%d/%b/%Y-%H:%M", local);
+      strftime(ftime, sizeof(ftime), "%d/%b/%Y-%H:%M", local);
 
    if(dotinuser && strchr(name,'_')) {
       subs(name,sizeof(name),"_",".");
    }
 
-   SARGgdImageStringFT(im,&brect[0],darkblue,font1,7,0.0,620,470,ftime);
-   if(strcmp(ShowSargInfo,"yes") == 0) SARGgdImageStringFT(im,&brect[0],darkblue,font1,10,0.0,257,15,"SARG, ");
-   SARGgdImageStringFT(im,&brect[0],darkblue,font1,10,0.0,300,15,Title);
+   SARGgdImageStringFT(im,&brect[0],darkblue,GraphFont,7,0.0,620,470,ftime);
+   if(ShowSargInfo) SARGgdImageStringFT(im,&brect[0],darkblue,GraphFont,10,0.0,257,15,"SARG, ");
+   SARGgdImageStringFT(im,&brect[0],darkblue,GraphFont,10,0.0,300,15,Title);
    sprintf(warea,"%s: %s",text[89],period);
-   SARGgdImageStringFT(im,&brect[0],darkblue,font1,9,0.0,300,27,warea);
+   SARGgdImageStringFT(im,&brect[0],darkblue,GraphFont,9,0.0,300,27,warea);
    sprintf(warea,"%s: %s",text[90],name);
-   SARGgdImageStringFT(im,&brect[0],darkblue,font1,9,0.0,300,38,warea);
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,418,"  50K");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,408,"250K");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,398,"500K");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,388,"   1M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,378,"   2M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,368,"   3M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,358,"   4M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,348,"   5M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,338,"   6M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,328,"   7M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,318,"   8M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,308,"   9M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,298," 10M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,288," 15M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,278," 20M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,268," 30M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,258," 40M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,248," 50M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,238," 60M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,228," 70M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,218," 80M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,208," 90M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,198,"100M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,188,"200M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,178,"300M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,168,"400M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,158,"500M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,148,"600M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,138,"700M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,128,"800M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,118,"900M");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23,108,"   1G");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23, 98,"   2G");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23, 88,"   3G");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23, 78,"   4G");
-   SARGgdImageStringFT(im,&brect[0],dimgray,font1,7,0.0,23, 68,"   5G");
-   SARGgdImageStringFT(im,&brect[0],black,font1,10,3.14/2,20,248,text[93]);
-   SARGgdImageStringFT(im,&brect[0],black,font1,10,0.0,330,460,text[127]);
+   SARGgdImageStringFT(im,&brect[0],darkblue,GraphFont,9,0.0,300,38,warea);
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,418,"  50K");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,408,"250K");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,398,"500K");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,388,"   1M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,378,"   2M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,368,"   3M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,358,"   4M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,348,"   5M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,338,"   6M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,328,"   7M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,318,"   8M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,308,"   9M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,298," 10M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,288," 15M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,278," 20M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,268," 30M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,258," 40M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,248," 50M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,238," 60M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,228," 70M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,218," 80M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,208," 90M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,198,"100M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,188,"200M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,178,"300M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,168,"400M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,158,"500M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,148,"600M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,138,"700M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,128,"800M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,118,"900M");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23,108,"   1G");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23, 98,"   2G");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23, 88,"   3G");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23, 78,"   4G");
+   SARGgdImageStringFT(im,&brect[0],dimgray,GraphFont,7,0.0,23, 68,"   5G");
+   if(strcmp(datetimeby,"bytes") == 0)
+      SARGgdImageStringFT(im,&brect[0],black,GraphFont,10,3.14/2,20,248,text[93]);
+   else
+      SARGgdImageStringFT(im,&brect[0],black,GraphFont,10,3.14/2,20,248,text[94]);
+   SARGgdImageStringFT(im,&brect[0],black,GraphFont,10,0.0,330,460,text[127]);
 
    if (snprintf(graph,sizeof(graph),"%s/%s/graph_day.png",dirname,user)>=sizeof(graph)) {
       fprintf(stderr, "SARG: user name too long for: %s/%s/graph_day.png\n",dirname,user);
@@ -394,9 +409,10 @@ void greport_day(const char *user)
       return;
    }
 
-   sprintf(csort,"sort -t\"/\" -k 2,2 -o \"%s\" \"%s\"",tmp5,wdirname);
    if(strcmp(DateFormat,"e") == 0)
       sprintf(csort,"sort -t\"/\" -k 1,1 -o \"%s\" \"%s\"",tmp5,wdirname);
+   else
+      sprintf(csort,"sort -t\"/\" -k 2,2 -o \"%s\" \"%s\"",tmp5,wdirname);
 
    cstatus=system(csort);
    if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
@@ -419,7 +435,7 @@ void greport_day(const char *user)
    if(strstr(wuser,"_") != 0)
       fixip(wuser);
 
-   if(strcmp(Ip2Name,"yes") == 0)
+   if(Ip2Name)
       if((str=(char *) strstr(name, ".")) != (char *) NULL) {
          if((str=(char *) strstr(str+1, ".")) != (char *) NULL)
             ip2name(wuser,sizeof(wuser));
@@ -428,38 +444,44 @@ void greport_day(const char *user)
    get_usertab_name(wuser,name,sizeof(name));
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if (getword(data,sizeof(data),buf,' ')<0 || getword(day,sizeof(day),data,'/')<0) {
+      fixendofline(buf);
+      getword_start(&gwarea,buf);
+      if (getword(data,sizeof(data),&gwarea,'\t')<0) {
+         printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp5);
+         exit(1);
+      }
+      getword_start(&gwarea1,data);
+      if (getword_atoll(&llday,&gwarea1,'/')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp5);
          exit(1);
       }
       if(strcmp(DateFormat,"u") == 0) {
-         if (getword(day,sizeof(day),data,'/')<0) {
+         if (getword_atoll(&llday,&gwarea1,'/')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp5);
             exit(1);
          }
       }
+      day=(int)llday;
       if(!count) {
-         strcpy(oday,day);
+         oday=day;
          count++;
       }
-      if (getword(bytes,sizeof(bytes),buf,' ')<0 || getword(bytes,sizeof(bytes),buf,' ')<0) {
+      if (getword_skip(20,&gwarea,'\t')<0 || getword_atoll(&bytes,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp5);
          exit(1);
       }
-      bytes[strlen(bytes)-1]='\0';
 
-      if(strcmp(oday,day) != 0) {
-         strcpy(warea,oday);
-         x1 = 44 +(atoi(oday) * 20);
+      if(oday!=day) {
+         x1 = 44 +(oday * 20);
          bar(x1,tot);
-         strcpy(oday,day);
+         oday=day;
          tot=0;
       }
-      tot=tot+my_atoll(bytes);
+      tot+=bytes;
    }
 
    if(tot) {
-      x1 = 44 +(atoi(day) * 20);
+      x1 = 44 +(day * 20);
       bar(x1,tot);
    }
 
@@ -471,7 +493,7 @@ void greport_day(const char *user)
    unlink(wdirname);
    unlink(tmp5);
 
-#endif
+#endif //HAVE_GD
 
    return;
 }
