@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                      orso@penguintech.com.br
- *                                                            1998, 2005
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,16 +36,19 @@ void htmlrel(void)
    long long int nnbytes=0, unbytes=0, tnbytes=0, totbytes=0, totbytes2=0;
    long long int totelap=0, totelap2=0, nnelap=0, unelap=0, tnelap=0;
    long long int incache=0, oucache=0, tnincache=0, tnoucache=0, twork=0, twork2=0;
+   long long int ltemp;
+   long long int ntotuser;
+   long long int userbytes, userelap;
    char arqin[MAXLEN], arqou[MAXLEN], arqper[MAXLEN], arqip[MAXLEN];
-   char nacc[20], nbytes[20], url[MAXLEN], tmsg[50], nelap[20], csort[MAXLEN];
+   char url[MAXLEN], tmsg[50], csort[MAXLEN];
    char period[MAXLEN], usuario[MAXLEN], wusuario[MAXLEN], u2[MAXLEN], duser[MAXLEN];
-   char userbytes[20], userelap[20], userurl[1024], userhora[9], userdia[9];
-   char user_ip[MAXLEN], olduserip[MAXLEN], tmp2[MAXLEN], tmp3[MAXLEN], incac[20], oucac[20];
+   char userurl[1024], userhora[9], userdia[9];
+   char user_ip[MAXLEN], olduserip[MAXLEN], tmp2[MAXLEN], tmp3[MAXLEN];
    char denied_report[255], name2[MAXLEN];
    //char ttd1[3], ttd2[3], ttd3[5], ttt1[3], ttt2[3], ttt3[3];
    char *str;
    char warea[MAXLEN];
-   char wtemp[MAXLEN], totuser[8];
+   char totuser[8];
    long long int tnacc=0, ttnacc=0, unacc=0;
    float perc=0, perc2=0, ouperc=0, inperc=0;
    char *s;
@@ -55,6 +59,7 @@ void htmlrel(void)
    char href2[MAXLEN];
    char urly[MAXLEN];
    char siteind[MAXLEN];
+   struct getwordstruct gwarea;
 
    if(strstr(ReportType,"users_sites") == 0) return;
 
@@ -88,23 +93,24 @@ void htmlrel(void)
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
       if(strstr(buf,"TOTAL") == 0) {
-         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+         getword_start(&gwarea,buf);
+         if (getword_skip(MAXLEN,&gwarea,'\t')<0 || getword_atoll(&ltemp,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqper);
             exit(1);
          }
-         ttnacc+=my_atoll(buf);
-         if (getword(wtemp,sizeof(wtemp),buf,' ')<0 || getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+         ttnacc+=ltemp;
+         if (getword_atoll(&ltemp,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqper);
             exit(1);
          }
-         totbytes+=my_atoll(wtemp);
-         if (getword(wtemp,sizeof(wtemp),buf,' ')<0 || getword(wtemp,sizeof(wtemp),buf,' ')<0 ||
-             getword(wtemp,sizeof(wtemp),buf,' ')<0 || getword(wtemp,sizeof(wtemp),buf,' ')<0 ||
-             getword(wtemp,sizeof(wtemp),buf,' ')<0) {
-            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqper);
+         totbytes+=ltemp;
+         if (getword_skip(MAXLEN,&gwarea,'\t')<0 || getword_skip(MAXLEN,&gwarea,'\t')<0 ||
+             getword_skip(MAXLEN,&gwarea,'\t')<0 || getword_skip(MAXLEN,&gwarea,'\t')<0 ||
+             getword_atoll(&ltemp,&gwarea,'\t')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file (%d).\n",arqper,__LINE__);
             exit(1);
          }
-         totelap+=my_atoll(wtemp);
+         totelap+=ltemp;
       }
    }
 
@@ -165,7 +171,7 @@ void htmlrel(void)
       strcpy(u2,usuario);
       if(userip)
          fixip(u2);
-      if(strcmp(Ip2Name,"yes") == 0)
+      if(Ip2Name)
          ip2name(u2,sizeof(u2));
 
       get_usertab_name(u2,name2,sizeof(name2));
@@ -180,32 +186,33 @@ void htmlrel(void)
       }
 
       while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+         getword_start(&gwarea,buf);
+         if (getword_atoll(&ltemp,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
             exit(1);
          }
-         tnacc+=my_atoll(wtemp);
-         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+         tnacc+=ltemp;
+         if (getword_atoll(&ltemp,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
             exit(1);
          }
-         tnbytes+=my_atoll(wtemp);
-         if (getword(wtemp,sizeof(wtemp),buf,' ')<0 || getword(wtemp,sizeof(wtemp),buf,' ')<0 ||
-             getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+         tnbytes+=ltemp;
+         if (getword_skip(MAXLEN,&gwarea,'\t')<0 || getword_skip(MAXLEN,&gwarea,'\t')<0 ||
+             getword_atoll(&ltemp,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
             exit(1);
          }
-         tnelap+=my_atoll(wtemp);
-         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
+         tnelap+=ltemp;
+         if (getword_atoll(&ltemp,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
             exit(1);
          }
-         tnincache+=my_atoll(wtemp);
-         if (getword(wtemp,sizeof(wtemp),buf,' ')<0) {
-            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arqin);
+         tnincache+=ltemp;
+         if (getword_atoll(&ltemp,&gwarea,'\n')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file (%d).\n",arqin,__LINE__);
             exit(1);
          }
-         tnoucache+=my_atoll(wtemp);
+         tnoucache+=ltemp;
       }
 
       rewind(fp_in);
@@ -288,32 +295,33 @@ void htmlrel(void)
       }
 
       while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-         if (getword(nacc,sizeof(nacc),buf,' ')<0) {
+         getword_start(&gwarea,buf);
+         if (getword_atoll(&twork,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken number of access in your %s file.\n",arqin);
             exit(1);
          }
-         if (getword(nbytes,sizeof(nbytes),buf,' ')<0) {
+         if (getword_atoll(&nnbytes,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken number of bytes in your %s file.\n",arqin);
             exit(1);
          }
-         if (getword(url,sizeof(url),buf,' ')<0) {
+         if (getword(url,sizeof(url),&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken url in your %s file.\n",arqin);
             exit(1);
          }
-         if (getword(tmsg,sizeof(tmsg),buf,' ')<0) {
+         if (getword(tmsg,sizeof(tmsg),&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken status in your %s file.\n",arqin);
             exit(1);
          }
-         if (getword(nelap,sizeof(nelap),buf,' ')<0) {
+         if (getword_atoll(&nnelap,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken elapsed time in your %s file.\n",arqin);
             exit(1);
          }
-         if (getword(incac,sizeof(incac),buf,' ')<0) {
+         if (getword_atoll(&incache,&gwarea,'\t')<0) {
             printf("SARG: Maybe you have a broken in cache column in your %s file.\n",arqin);
             exit(1);
          }
-         if (getword(oucac,sizeof(oucac),buf,' ')<0) {
-            printf("SARG: Maybe you have a broken not in cache column in your %s file.\n",arqin);
+         if (getword_atoll(&oucache,&gwarea,'\n')<0) {
+            printf("SARG: Maybe you have a broken not in cache column in your %s file (%d).\n",arqin,__LINE__);
             exit(1);
          }
 
@@ -321,54 +329,44 @@ void htmlrel(void)
             sprintf(tmsg,"<td class=\"data\">%s</td>",text[46]);
          else bzero(tmsg, 50);
 
-         nnbytes=my_atoll(nbytes);
-         nnelap=my_atoll(nelap);
-         incache=my_atoll(incac);
-         oucache=my_atoll(oucac);
-
-         if(nnbytes) {
-            perc=nnbytes * 100;
-            perc=perc / tnbytes;
+         if(tnbytes) {
+            perc=nnbytes * 100. / tnbytes;
          } else {
             perc=0;
-            ouperc=0;
          }
 
-         if(nnelap) {
-            perc2=nnelap * 100;
-            perc2=perc2 / tnelap;
+         if(tnelap) {
+            perc2=nnelap * 100. / tnelap;
          } else perc2=0;
 
-         if(incache) {
-            inperc=incache * 100;
-            inperc=inperc / nnbytes;
+         if(nnbytes) {
+            inperc=incache * 100. / nnbytes;
          } else inperc=0;
 
-         if(oucache) {
-            ouperc=oucache * 100;
-            ouperc=ouperc / nnbytes;
+         if(nnbytes) {
+            ouperc=oucache * 100. / nnbytes;
          } else ouperc=0;
 
-         twork=my_atoll(nacc);
          strcpy(wwork1,fixnum(twork,1));
          strcpy(wwork2,fixnum(nnbytes,1));
          strcpy(wwork3,fixnum2(nnelap,1));
 
-         if(strcmp(LongUrl,"yes") != 0) {
+         if(!LongUrl) {
             char *endofhost=strchr(url,'/');
             if (endofhost) *endofhost='\0';
             strcpy(urly,url);
          } else {
             strcpy(urly,url);
             url_module(url, module);
-            if (getword(warea,sizeof(warea),url,'/')<0) {
-               printf("SARG: Maybe you have a broken record or garbage in the url of the %s file.\n",arqin);
-               exit(1);
+            str=strchr(url,'/');
+            if (str){
+               *str='\0';
+               strcat(url,"...");
+               strcat(url,module);
             }
-            sprintf(url,"%s...%s",warea,module);
          }
 
-         if(strcmp(Privacy,"yes") == 0)
+         if(Privacy)
             sprintf(httplink,"<span style=\"color:%s;\">%s</span>",PrivacyStringColor,PrivacyString);
          else {
             if(strcmp(IndexTree,"date") == 0)
@@ -389,7 +387,7 @@ void htmlrel(void)
             str=siteind;
             for(z1=0; str[z1]; z1++) {
                if(str[z1]=='?' || str[z1]=='-' || str[z1]=='.' || str[z1]==':' || str[z1]=='/' || str[z1]=='\\' || str[z1]=='*' ||
-                  str[z1]=='\'' || str[z1]=='\"' || str[z1]=='$')
+                  str[z1]=='\'' || str[z1]=='\"' || str[z1]=='$' || str[z1]=='@')
                   str[z1]='_';
             }
             sprintf(href2,"<a href=\"tt%s-%s.html\"><img src=\"%s/datetime.png\" border=\"0\" title=\"%s %s\" alt=\"T\"></a>",usuario,siteind,tmp6,ltext110,text[55]);
@@ -484,28 +482,29 @@ void htmlrel(void)
             olduserip[0]='\0';
 
             while(fgets(buf,sizeof(buf),fp_ip)) {
-               if (getword(user_ip,sizeof(user_ip),buf,' ')<0) {
+               getword_start(&gwarea,buf);
+               if (getword(user_ip,sizeof(user_ip),&gwarea,'\t')<0) {
                   printf("SARG: Maybe you have a broken user IP in your %s file.\n",tmp3);
                   exit(1);
                }
-               if (getword(userurl,sizeof(userurl),buf,' ')<0) {
+               if (getword(userurl,sizeof(userurl),&gwarea,'\t')<0) {
                   printf("SARG: Maybe you have a broken user URL in your %s file.\n",tmp3);
                   exit(1);
                }
-               if (getword(userdia,sizeof(userdia),buf,' ')<0) {
+               if (getword(userdia,sizeof(userdia),&gwarea,'\t')<0) {
                   printf("SARG: Maybe you have a broken day in your %s file.\n",tmp3);
                   exit(1);
                }
-               if (getword(userhora,sizeof(userhora),buf,' ')<0) {
+               if (getword(userhora,sizeof(userhora),&gwarea,'\t')<0) {
                   printf("SARG: Maybe you have a broken time in your %s file.\n",tmp3);
                   exit(1);
                }
-               if (getword(userbytes,sizeof(userbytes),buf,' ')<0) {
+               if (getword_atoll(&userbytes,&gwarea,'\t')<0) {
                   printf("SARG: Maybe you have a broken size in your %s file.\n",tmp3);
                   exit(1);
                }
-               if (getword(userelap,sizeof(userelap),buf,' ')<0) {
-                  printf("SARG: Maybe you have a broken elapsed time in your %s file.\n",tmp3);
+               if (getword_atoll(&userelap,&gwarea,'\t')<0) {
+                  printf("SARG: Maybe you have a broken elapsed time in your %s file (%d).\n",tmp3,__LINE__);
                   exit(1);
                }
                if(strcmp(user_ip,olduserip) != 0) {
@@ -519,8 +518,8 @@ void htmlrel(void)
                   unelap=0;
                }
 
-               unbytes=unbytes+my_atoll(userbytes);
-               unelap=unelap+my_atoll(userelap);
+               unbytes=unbytes+userbytes;
+               unelap=unelap+userelap;
             }
 
             fclose(fp_ip);
@@ -543,24 +542,20 @@ void htmlrel(void)
          unlink(arqip);
       unlink(arqin);
 
-      if(tnbytes) {
-         perc=totbytes / 100;
-         perc=tnbytes / perc;
+      if(totbytes) {
+         perc=tnbytes *100. / totbytes;
       } else perc=0;
 
-      if(tnelap) {
-         perc2=totelap / 100;
-         perc2=tnelap / perc2;
+      if(totelap) {
+         perc2=tnelap *100. / totelap;
       } else perc2=0;
 
       if(tnoucache) {
-         ouperc=tnoucache * 100;
-         ouperc=ouperc / tnbytes;
+         ouperc=tnoucache * 100. / tnbytes;
       } else ouperc=0;
 
       if(tnincache) {
-         inperc=tnincache * 100;
-         inperc=inperc / tnbytes;
+         inperc=tnincache * 100. / tnbytes;
       } else inperc=0;
 
       sprintf(wwork1,"%s",fixnum(tnacc,1));
@@ -623,8 +618,8 @@ void htmlrel(void)
 
       fclose(fp_in);
 
-      if(atoi(PerUserLimit) > 0) {
-         if(tnbytes > (atoi(PerUserLimit)*1000000)) {
+      if(PerUserLimit > 0) {
+         if(tnbytes > (PerUserLimit*1000000)) {
             limit_flag=0;
             if(access(PerUserLimitFile, R_OK) == 0) {
                if((fp_usr = fopen(PerUserLimitFile, "r")) == 0) {
@@ -651,7 +646,7 @@ void htmlrel(void)
                fclose(fp_usr);
 
                if(debug)
-                  debuga("%s %s %s (%s MB). %s %s",text[32],u2,text[74],PerUserLimit,text[75],PerUserLimitFile);
+                  debuga("%s %s %s (%d MB). %s %s",text[32],u2,text[74],PerUserLimit,text[75],PerUserLimitFile);
             }
          }
       }
@@ -670,26 +665,26 @@ void htmlrel(void)
             exit(1);
          }
 
-         if (!fgets(totuser,8,fp_in)) {
+         if (!fgets(totuser,sizeof(totuser),fp_in)) {
             fprintf(stderr,"SARG: (html11) read error in %s\n",arqper);
             exit(1);
          }
          fclose(fp_in);
 
-         totbytes2=totbytes/my_atoll(totuser);
-         totelap2=totelap/my_atoll(totuser);
+         ntotuser=my_atoll(totuser);
+         if (ntotuser<=0) ntotuser=1;
+         totbytes2=totbytes/ntotuser;
+         totelap2=totelap/ntotuser;
 
-         if(totbytes2) {
-            perc = totbytes / 100;
-            perc = totbytes2 / perc;
+         if(totbytes) {
+            perc = totbytes2 * 100. / totbytes;
          } else perc=0;
 
-         if(totelap2) {
-            perc2 = totelap / 100;
-            perc2 = totelap2 / perc2;
+         if(totelap) {
+            perc2 = totelap2 * 100. / totelap;
          } else perc2=0;
 
-         twork2=my_atoll(totuser);
+         twork2=ntotuser;
          twork=ttnacc/twork2;
 
          strcpy(wwork1,fixnum(twork,1));
