@@ -94,6 +94,12 @@ gdPoint points[4];
 #ifdef HAVE_WINSOCK_H
 #include <winsock.h>
 #endif
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
+#ifdef HAVE_EXECINFO_H
+#include <execinfo.h>
+#endif
 
 #if defined(HAVE_FOPEN64)
 #define _FILE_OFFSET_BITS 64
@@ -125,23 +131,19 @@ int mkstemps(char *template, int suffixlen);
 #endif /*__MINGW32__*/
 
 #define MAXLEN 20000
-long long int my_atoll (const char *nptr);
+#define MAXLOGS 255
 
 FILE *fp_tt;
 
 char outdir[MAXLEN];
 char dirname[MAXLEN];
 char buf[MAXLEN];
-char url[MAXLEN];
-char urly[MAXLEN];
 char user[MAXLEN];
 char period[MAXLEN];
 char msg[1024];
-char per_hour[128];
 char code[MAXLEN];
 char code2[MAXLEN];
 char tmp[MAXLEN];
-char tmp2[MAXLEN];
 char tmp3[MAXLEN];
 char tmp4[MAXLEN];
 char tmp5[MAXLEN];
@@ -157,20 +159,19 @@ char href2[MAXLEN];
 char href3[MAXLEN];
 char df[20];
 char day[3], month[4], year[5];
-char nmonth[30];
 char ltext110[50];
 char cdfrom[30];
 char cduntil[30];
-char LastLog[5];
-char RemoveTempFiles[4];
+int LastLog;
+int RemoveTempFiles;
 char ReplaceIndex[256];
 char Index[20];
-char OverwriteReport[4];
+int OverwriteReport;
 char u2[255];
 char RecordsWithoutUser[20];
-char UseComma[4];
+int UseComma;
 char MailUtility[PATH_MAX];
-char TopSitesNum[20];
+int TopSitesNum;
 int TopUsersNum;
 char ExcludeCodes[256];
 char TopsitesSortField[15];
@@ -179,11 +180,13 @@ char ReportType[255];
 char UserTabFile[255];
 char warea[MAXLEN];
 char name[MAXLEN];
-char LongUrl[20];
-char Ip2Name[20];
+int LongUrl;
+int Ip2Name;
 char language[255];
 char bufy[MAXLEN];
-char AccessLog[MAXLEN];
+int NAccessLog;
+char AccessLog[MAXLOGS][MAXLEN];
+int AccessLogFromCmdLine;
 char Title[MAXLEN];
 char BgColor[MAXLEN];
 char BgImage[MAXLEN];
@@ -213,42 +216,41 @@ char ExcludeHosts[255];
 char ExcludeUsers[255];
 char DateFormat[2];
 char PerUserLimitFile[255];
-char PerUserLimit[20];
-char UserIp[5];
+int PerUserLimit;
+int UserIp;
 char MaxElapsed[255];
 char datetimeby[10];
 char csort[255];
 char CharSet[255];
 char UserInvalidChar[255];
-char Graphs[5];
+int Graphs;
 char GraphDaysBytesBarColor[255];
-char Privacy[10];
+int Privacy;
 char PrivacyString[255];
 char PrivacyStringColor[30];
 char IncludeUsers[MAXLEN];
 char ExcludeString[MAXLEN];
-char SuccessfulMsg[5];
+int SuccessfulMsg;
 char TopUserFields[255];
 char UserReportFields[255];
 char DataFile[MAXLEN];
 char DataFileDelimiter[3];
 char DataFileFields[MAXLEN];
 char DataFileUrl[20];
-char SiteUserTimeDateType[10];
-char ShowReadStatistics[5];
+int ShowReadStatistics;
 char IndexSortOrder[5];
 char DansGuardianConf[MAXLEN];
-char DansguardianIgnoreDate[10];
+int DansguardianIgnoreDate;
 char SquidGuardConf[MAXLEN];
 char SquidGuarddbHome[255];
 char SquidGuardLogFormat[MAXLEN];
 char SquidGuardLogAlternate[MAXLEN];
-char SquidguardIgnoreDate[10];
-char ShowSargInfo[5];
-char BytesInSitesUsersReport[10];
-char ShowSargLogo[5];
+int SquidguardIgnoreDate;
+int ShowSargInfo;
+int BytesInSitesUsersReport;
+int ShowSargLogo;
 char ParsedOutputLog[MAXLEN];
-char ParsedOutputLogCompress[255];
+char ParsedOutputLogCompress[512];
 char DisplayedValues[20];
 char HeaderFontSize[5];
 char TitleFontSize[5];
@@ -268,12 +270,11 @@ char hbc8[30];
 char hbc9[30];
 char hbc10[30];
 char IndexTree[10];
-char UserAuthentication[10];
+int UserAuthentication;
 char AuthUserFile[255];
 char AuthName[512];
 char AuthType[255];
 char Require[512];
-char DownloadSuffix[MAXLEN];
 char *userfile;
 char *str;
 char *str2;
@@ -292,7 +293,6 @@ char val11[MAXLEN];
 char wwork1[MAXLEN];
 char wwork2[MAXLEN];
 char wwork3[MAXLEN];
-char ftime[128];
 char mask[MAXLEN];
 char httplink[MAXLEN];
 char html_old[MAXLEN];
@@ -311,6 +311,7 @@ char ImageFile[255];
 char tbuf[128];
 char ip[25];
 char RealtimeUnauthRec[15];
+char GraphFont[MAXLEN];
 
 int  idate;
 int  smartfilter;
