@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -27,7 +28,7 @@
 #include "include/defs.h"
 
 void ip2name(char *ip,int ip_len)
-{ 
+{
    u_long addr;
    struct hostent *hp;
    char **p;
@@ -51,7 +52,7 @@ void ip2name(char *ip,int ip_len)
 }
 
 void name2ip(char *name)
-{ 
+{
    struct in_addr ia;
    struct hostent *hp;
    char *port;
@@ -59,22 +60,23 @@ void name2ip(char *name)
    char n2[4];
    char n3[4];
    char n4[4];
+   struct getwordstruct gwarea;
 
    port=strchr(name,':');
    if (port) *port=0;
 
    if((hp=gethostbyname(name))==NULL)
       return;
-      
+
    memcpy(&ia.s_addr,hp->h_addr_list[0],sizeof(ia.s_addr));
    ia.s_addr=ntohl(ia.s_addr);
-   strcpy(name,inet_ntoa(ia));
-   if (getword(n4,sizeof(n4),name,'.')<0 || getword(n3,sizeof(n3),name,'.')<0 ||
-         getword(n2,sizeof(n2),name,'.')<0 || getword(n1,sizeof(n1),name,0)<0) {
-      printf("SARG: Maybe you have a broken record or garbage in your %s ip address.\n",name);
+   getword_start(&gwarea,inet_ntoa(ia));
+   if (getword(n4,sizeof(n4),&gwarea,'.')<0 || getword(n3,sizeof(n3),&gwarea,'.')<0 ||
+         getword(n2,sizeof(n2),&gwarea,'.')<0 || getword(n1,sizeof(n1),&gwarea,0)<0) {
+      printf("SARG: Maybe you have a broken record or garbage in your %s ip address.\n",gwarea.beginning);
       exit(1);
    }
    sprintf(name,"%s.%s.%s.%s",n1,n2,n3,n4);
 
    return;
-} 
+}
