@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                      orso@penguintech.com.br
- *                                                            1998, 2005
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -26,8 +27,8 @@
 #include "include/conf.h"
 #include "include/defs.h"
 
-int getdata(char *, FILE *);
-void datashow(char *);
+static int getdata(char*, FILE*);
+static void datashow(const char *);
 static void getlog(void);
 static void header(void);
 
@@ -64,7 +65,7 @@ static void getlog(void)
        exit(1);
    }
 
-   sprintf(cmd,"tail -%d %s",realtime_access_log_lines,AccessLog);
+   sprintf(cmd,"tail -%d %s",realtime_access_log_lines,AccessLog[0]);
    fp = popen(cmd, "r");
    while(fgets(buf,sizeof(buf),fp) != NULL )
       if (getdata(buf,tmp)<0) {
@@ -85,64 +86,69 @@ static void getlog(void)
    datashow(template2);
 }
 
-int getdata(char *rec, FILE *ftmp)
+static int getdata(char *rec, FILE *ftmp)
 {
    time_t tt;
    struct tm *t;
+   char tbuf[128];
+   char warea[MAXLEN];
+   char url[MAXLEN];
+   struct getwordstruct gwarea;
 
-   if (getword3(dat,sizeof(dat),rec,' ')<0) {
+   getword_start(&gwarea,rec);
+   if (getword(dat,sizeof(dat),&gwarea,' ')<0) {
       fprintf(stderr,"SARG: The time stamp at column 1 is too long.\n");
       return(-1);
    }
-   if (getword3(warea,sizeof(warea),rec,' ')<0) {
+   if (getword(warea,sizeof(warea),&gwarea,' ')<0) {
       fprintf(stderr,"SARG: The connection duration at column 2 is too long.\n");
       return(-1);
    }
-   while(strcmp(warea,"") == 0 && strlen(rec) > 0)
-      if (getword3(warea,sizeof(warea),rec,' ')<0) {
+   while(strcmp(warea,"") == 0 && gwarea.current[0] != '\0')
+      if (getword(warea,sizeof(warea),&gwarea,' ')<0) {
          return(-1);
       }
-   if (getword3(ip,sizeof(ip),rec,' ')<0) {
+   if (getword(ip,sizeof(ip),&gwarea,' ')<0) {
       fprintf(stderr,"SARG: The IP address at column 3 is too long.\n");
       return(-1);
    }
-   if (getword3(warea,sizeof(warea),rec,' ')<0) {
+   if (getword_skip(MAXLEN,&gwarea,' ')<0) {
       fprintf(stderr,"SARG: The status at column 4 is too long.\n");
       return(-1);
    }
-   if (getword3(warea,sizeof(warea),rec,' ')<0) {
+   if (getword_skip(MAXLEN,&gwarea,' ')<0) {
       fprintf(stderr,"SARG: The size at column 5 is too long.\n");
       return(-1);
    }
-   if (getword3(typ,sizeof(typ),rec,' ')<0) {
+   if (getword(typ,sizeof(typ),&gwarea,' ')<0) {
       fprintf(stderr,"SARG: The action at column 6 is too long.\n");
       return(-1);
    }
    if(strncmp(typ,"CONNECT",7) == 0) {
-      if (getword3(url,sizeof(url),rec,' ')<0) {
+      if (getword(url,sizeof(url),&gwarea,' ')<0) {
          return(-1);
       }
-      if (getword3(user,sizeof(user),rec,' ')<0) {
+      if (getword(user,sizeof(user),&gwarea,' ')<0) {
          return(-1);
       }
    }else {
-      if (getword3(url,sizeof(url),rec,'/')<0) {
+      if (getword_skip(MAXLEN,&gwarea,'/')<0) {
          fprintf(stderr,"SARG: The URL at column 7 is too long.\n");
          return(-1);
       }
-      if (getword3(url,sizeof(url),rec,'/')<0) {
+      if (getword_skip(MAXLEN,&gwarea,'/')<0) {
          fprintf(stderr,"SARG: The URL at column 7 is too long.\n");
          return(-1);
       }
-      if (getword3(url,sizeof(url),rec,'/')<0) {
+      if (getword(url,sizeof(url),&gwarea,'/')<0) {
          fprintf(stderr,"SARG: The URL at column 7 is too long.\n");
          return(-1);
       }
-      if (getword3(user,sizeof(user),rec,' ')<0) {
+      if (getword_skip(MAXLEN,&gwarea,' ')<0) {
          fprintf(stderr,"SARG: The data at column 8 is too long.\n");
          return(-1);
       }
-      if (getword3(user,sizeof(user),rec,' ')<0) {
+      if (getword(user,sizeof(user),&gwarea,' ')<0) {
          fprintf(stderr,"SARG: The user at column 9 is too long.\n");
          return(-1);
       }
@@ -154,18 +160,20 @@ int getdata(char *rec, FILE *ftmp)
    tt=atoi(dat);
    t=localtime(&tt);
    if(strncmp(DateFormat,"u",1) == 0)
-      strftime(tbuf, 127, "%Y-%m-%d %H:%M", t);
+      strftime(tbuf, sizeof(tbuf), "%Y-%m-%d\t%H:%M", t);
    else if(strncmp(DateFormat,"e",1) == 0)
-      strftime(tbuf, 127, "%d-%m-%Y %H:%M", t);
+      strftime(tbuf, sizeof(tbuf), "%d-%m-%Y\t%H:%M", t);
 
-   fprintf(ftmp,"%s %s %s %s %s\n",tbuf,ip,user,url,typ);
+   fprintf(ftmp,"%s\t%s\t%s\t%s\t%s\n",tbuf,ip,user,url,typ);
    return(0);
 }
 
-void datashow(char *tmp)
+static void datashow(const char *tmp)
 {
    FILE *fin;
    char buf[MAXLEN];
+   char url[MAXLEN];
+   struct getwordstruct gwarea;
 
    if((fin=fopen(tmp,"r"))==NULL) {
       fprintf(stderr, "SARG: (realtime) open error %s - %s\n",tmp,strerror(errno));
@@ -174,30 +182,31 @@ void datashow(char *tmp)
 
    header();
 
-   while(fgets(buf, MAXLEN, fin)) {
-      buf[strlen(buf)-1]='\0';
-      if (getword3(dat,sizeof(dat),buf,' ')<0) {
+   while(fgets(buf, sizeof(buf), fin)) {
+      fixendofline(buf);
+      getword_start(&gwarea,buf);
+      if (getword(dat,sizeof(dat),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
          exit(1);
       }
-      if (getword3(tim,sizeof(tim),buf,' ')<0) {
+      if (getword(tim,sizeof(tim),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
          exit(1);
       }
-      if (getword3(ip,sizeof(ip),buf,' ')<0) {
+      if (getword(ip,sizeof(ip),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
          exit(1);
       }
-      if (getword3(user,sizeof(user),buf,' ')<0) {
+      if (getword(user,sizeof(user),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
          exit(1);
       }
       if(strlen(dat) < 3 || strlen(user) < 1) continue;
-      if (getword3(url,sizeof(url),buf,' ')<0) {
+      if (getword(url,sizeof(url),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
          exit(1);
       }
-      if (getword3(typ,sizeof(typ),buf,' ')<0) {
+      if (getword(typ,sizeof(typ),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp);
          exit(1);
       }
@@ -207,8 +216,10 @@ void datashow(char *tmp)
       if(strcmp(ouser,user) == 0 && strcmp(ourl,url) == 0)
          continue;
 
+      if(userip)
+         strcpy(user,ip);
       strcpy(u2,user);
-      if(strcmp(Ip2Name,"yes") == 0)
+      if(Ip2Name)
          ip2name(u2,sizeof(u2));
       get_usertab_name(u2,name,sizeof(name));
 
