@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,8 +37,6 @@ void report_day(const char *user)
    char data[20];
    char odata[20];
    char hour[20];
-   char elap[20];
-   char oelap[20];
    char html[8000];
    char arqout[MAXLEN];
    char wdirname[MAXLEN];
@@ -45,12 +44,14 @@ void report_day(const char *user)
    char c[ 24 ][20];
    int  count=0;
    int  ihour=0;
-   long long int v[ 24 ] = { 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 
+   long long int elap;
+   long long int v[ 24 ] = { 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
 			     0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L };
-   long long int t[ 24 ] = { 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 
+   long long int t[ 24 ] = { 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L,
 			     0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L };
    long long int tt=0, ttt=0;
    int i;
+   struct getwordstruct gwarea;
 
    sprintf(arqout,"%s/%s/d%s.html",dirname,user,user);
    sprintf(wdirname,"%s/%s.day",tmp,user);
@@ -92,7 +93,7 @@ void report_day(const char *user)
    if(strstr(wuser,"_") != 0)
       fixip(wuser);
 
-   if(strcmp(Ip2Name,"yes") == 0)
+   if(Ip2Name)
       if((str=(char *) strstr(name, ".")) != (char *) NULL) {
          if((str=(char *) strstr(str+1, ".")) != (char *) NULL)
             ip2name(wuser,sizeof(wuser));
@@ -124,7 +125,9 @@ void report_day(const char *user)
      "<td class=\"header3\">%s<br>%s</td></tr>\n", text[107], html );
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if (getword(data,sizeof(data),buf,' ')<0) {
+      fixendofline(buf);
+      getword_start(&gwarea,buf);
+      if (getword(data,sizeof(data),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdirname);
          exit(1);
       }
@@ -133,15 +136,12 @@ void report_day(const char *user)
          count++;
       }
 
-      if (getword(hour,sizeof(hour),buf,' ')<0 || getword(elap,sizeof(elap),buf,' ')<0) {
+      if (getword(hour,sizeof(hour),&gwarea,'\t')<0 || getword_atoll(&elap,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wdirname);
          exit(1);
       }
-      elap[strlen(elap)-1]='\0';
 
       if(strcmp(data,odata) != 0) {
-         strcpy(oelap,elap);
-
          for( i = 0; i < hours.len; i++ )
             sprintf(c[ hours.list[ i ] ],"%s",fixtime(v[ hours.list[ i ] ]));
 
@@ -157,15 +157,14 @@ void report_day(const char *user)
          tt=0;
          for( i = 0; i < hours.len; i++ ) v[ hours.list[ i ] ]=0;
          strcpy(odata,data);
-         strcpy(elap,oelap);
       }
 
       ihour=atoi(hour);
 
-      v[ ihour ]+=my_atoll(elap);
-      tt+=my_atoll(elap);
-      t[ ihour ]+=my_atoll(elap);
-      ttt+=my_atoll(elap);
+      v[ ihour ]+=elap;
+      tt+=elap;
+      t[ ihour ]+=elap;
+      ttt+=elap;
 
    }
 
