@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -28,24 +29,24 @@
 
 static void maketmp(const char *user, const char *dirname, int debug, int indexonly);
 static void maketmp_hour(const char *user, const char *dirname, int indexonly);
-static void gravatmp_hora(const char *dirname, const char *user, const char *data, const char *hora, const char *elap, const char *accbytes, int indexonly);
+static void gravatmp_hora(const char *dirname, const char *user, const char *data, const char *hora, long long int elap, long long int accbytes, int indexonly);
 static void gravatmpf(const char *oldaccuser, const char *dirname, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, int indexonly, long long int incache, long long int oucache);
-static void gravaporuser(const char *user, const char *dirname, const char *url, const char *ip, const char *data, const char *hora, const char *tam, const char *elap, int indexonly);
-static void gravager(char *dirname, const char *user, long long int nacc, const char *url, long long int nbytes, const char *ip, const char *hora, const char *dia, long long int nelap, long long int incache, long long int oucache);
+static void gravaporuser(const char *user, const char *dirname, const char *url, const char *ip, const char *data, const char *hora, long long int tam, long long int elap, int indexonly);
+static void gravager(FILE *fp_gen, const char *user, long long int nacc, const char *url, long long int nbytes, const char *ip, const char *hora, const char *dia, long long int nelap, long long int incache, long long int oucache);
 static void grava_SmartFilter(const char *dirname, const char *user, const char *ip, const char *data, const char *hora, const char *url, const char *smart);
 
 void gerarel(void)
 {
 
    FILE *fp_in;
+   FILE *fp_gen;
 
-   char accdia[11], acchora[9], accuser[MAXLEN], accip[MAXLEN], accurl[MAXLEN], accbytes[12], accelap[10];
-   char oldaccdia[11], oldacchora[9], oldaccip[MAXLEN], wdir[MAXLEN], per1[MAXLEN];
+   char accdia[11], acchora[9], accuser[MAXLEN], accip[MAXLEN], accurl[MAXLEN];
+   char oldaccdia[11], oldacchora[9], oldaccip[MAXLEN];
    char wdirname[MAXLEN], oldurl[MAXLEN], oldaccuser[MAXLEN];
-   char olduser[MAXLEN], oldmsg[50], acccode[MAXLEN/2 - 1], oldaccelap[10], oldacccode[MAXLEN/2 - 1], user[MAXLEN];
+   char olduser[MAXLEN], oldmsg[50], acccode[MAXLEN/2 - 1], oldacccode[MAXLEN/2 - 1], user[MAXLEN];
    char ipantes[MAXLEN], nameantes[MAXLEN];
    char accsmart[MAXLEN];
-   char wcrc[MAXLEN/2 -1];
    char crc2[MAXLEN/2 -1];
    long long int nbytes=0;
    long long int nelap=0;
@@ -53,31 +54,36 @@ void gerarel(void)
    long long int rtotal=0;
    long long int incache=0;
    long long int oucache=0;
-   char *s;
+   long long int accbytes, accelap;
+   char *str;
    DIR *dirp;
    struct dirent *direntp;
    const char logext[]=".log";
    int dlen;
    char siteind[MAXLEN];
+   struct getwordstruct gwarea;
 
    ipantes[0]='\0';
    nameantes[0]='\0';
    smartfilter=0;
 
    sprintf(dirname, "%s%s", outdir, period);
-   sprintf(wdir, "%s%s", outdir, period);
-   strcpy(per1,period);
-   vrfydir(wdir, per1, addr, site, us, email);
+   vrfydir(period, addr, site, us, email);
 
    if(debugz){
       debugaz("dirname",dirname);
-      debugaz("wdir",wdir);
    }
 
    gperiod(dirname,period);
 
    if(UserAgentLog[0] != '\0' && email[0] == '\0') useragent();
 
+   sprintf(wdirname,"%s/sarg-general",dirname);
+   if((fp_gen=MY_FOPEN(wdirname,"a"))==NULL){
+      fprintf(stderr, "SARG: (report) %s: %s\n",text[45],wdirname);
+      exit(1);
+   }
+
    olduser[0]='\0';
    strncat(tmp,"/sarg",5);
 
@@ -112,7 +118,7 @@ void gerarel(void)
       maketmp_hour(user,tmp,indexonly);
 
       strcpy(u2,user);
-      if(strcmp(Ip2Name,"yes") == 0)
+      if(Ip2Name)
          ip2name(u2,sizeof(u2));
       get_usertab_name(u2,name,sizeof(name));
 
@@ -124,17 +130,21 @@ void gerarel(void)
       bzero(html_old, MAXLEN);
 
       while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-         if (getword(accdia,sizeof(accdia),buf,' ')<0 || getword(acchora,sizeof(acchora),buf,' ')<0 ||
-             getword(accuser,sizeof(accuser),buf,' ')<0 || getword(accip,sizeof(accip),buf,' ')<0 ||
-             getword(accurl,sizeof(accurl),buf,' ')<0 || getword(accbytes,sizeof(accbytes),buf,' ')<0 ||
-             getword(acccode,sizeof(acccode),buf,' ')<0) {
-            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp3);
+         getword_start(&gwarea,buf);
+         if (getword(accdia,sizeof(accdia),&gwarea,'\t')<0 || getword(acchora,sizeof(acchora),&gwarea,'\t')<0 ||
+             getword(accuser,sizeof(accuser),&gwarea,'\t')<0 || getword(accip,sizeof(accip),&gwarea,'\t')<0 ||
+             getword(accurl,sizeof(accurl),&gwarea,'\t')<0 || getword_atoll(&accbytes,&gwarea,'\t')<0 ||
+             getword(acccode,sizeof(acccode),&gwarea,'\t')<0) {
+            printf("SARG: Maybe you have a broken record or garbage in your %s file (%d).\n",tmp3,__LINE__);
             exit(1);
          }
          if(strncmp(acccode,"TCP_DENIED/407",14) == 0) continue;
-         if (getword(accelap,sizeof(accelap),buf,' ')<0 || getword(accsmart,sizeof(accsmart),buf,' ')<0 ||
-             getword(accsmart,sizeof(accsmart),buf,'"')<0) {
-            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp3);
+         if (getword_atoll(&accelap,&gwarea,'\t')<0 || getword_skip(20000,&gwarea,'\t')<0) {
+            printf("SARG: Maybe you have a broken elapsed time in your %s file (%d).\n",tmp3,__LINE__);
+            exit(1);
+         }
+         if (getword(accsmart,sizeof(accsmart),&gwarea,'"')<0) {
+            printf("SARG: Maybe you have a broken smart info in your %s file (%d).\n",tmp3,__LINE__);
             exit(1);
          }
 
@@ -144,7 +154,7 @@ void gerarel(void)
             grava_SmartFilter(wdirname,accuser,accip,accdia,acchora,accurl,accsmart);
          }
 
-         if(strcmp(Ip2Name,"yes") == 0) {
+         if(Ip2Name) {
             if(strcmp(accip,ipantes) != 0) {
                strcpy(ipantes,accip);
                ip2name(accip,sizeof(accip));
@@ -163,7 +173,6 @@ void gerarel(void)
          if(!rtotal){
             strcpy(oldurl,accurl);
             strcpy(oldacccode,acccode);
-            strcpy(oldaccelap,accelap);
             strcpy(oldaccuser,accuser);
             strcpy(oldaccip,accip);
             strcpy(oldaccdia,accdia);
@@ -176,8 +185,8 @@ void gerarel(void)
                strcpy(oldmsg,"OK");
                if(strstr(oldacccode,"DENIED") != 0)
                   sprintf(oldmsg,"%s",text[46]);
-               gravatmp(oldaccuser,wdirname,oldurl,nacc,nbytes,oldmsg,nelap,indexonly,incache,oucache);
-               gravager(wdirname,oldaccuser,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
+               gravatmp(oldaccuser,oldurl,nacc,nbytes,oldmsg,nelap,indexonly,incache,oucache);
+               gravager(fp_gen,oldaccuser,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
                nacc=0;
                nbytes=0;
                nelap=0;
@@ -189,10 +198,8 @@ void gerarel(void)
                strcpy(oldmsg,"OK");
                if(strstr(oldacccode,"DENIED") != 0)
                   sprintf(oldmsg,"%s",text[46]);
-               strcpy(wdirname,dirname);
-               gravatmp(oldaccuser,wdirname,oldurl,nacc,nbytes,oldmsg,nelap,indexonly,incache,oucache);
-               strcpy(wdirname,dirname);
-               gravager(wdirname,oldaccuser,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
+               gravatmp(oldaccuser,oldurl,nacc,nbytes,oldmsg,nelap,indexonly,incache,oucache);
+               gravager(fp_gen,oldaccuser,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
                nacc=0;
                nbytes=0;
                nelap=0;
@@ -203,8 +210,8 @@ void gerarel(void)
             }
          }
          nacc++;
-         nbytes+=my_atoll(accbytes);
-         nelap+=my_atoll(accelap);
+         nbytes+=accbytes;
+         nelap+=accelap;
 
          if(strstr(ReportType,"site_user_time_date") != 0) {
             if(!ttopen) {
@@ -212,7 +219,7 @@ void gerarel(void)
                strcpy(siteind,accurl);
                for(str=siteind; *str; str++) {
                   if(*str=='?' || *str=='-' || *str=='.' || *str==':' || *str=='/' || *str=='\\' || *str=='*' ||
-                     *str=='\'' || *str=='\"' || *str=='$')
+                     *str=='\'' || *str=='\"' || *str=='$' || *str=='@')
                      *str='_';
                }
                sprintf(arqtt,"%s/%s",dirname,accuser);
@@ -229,7 +236,7 @@ void gerarel(void)
                ttopen=1;
 
                /*
-               if(strcmp(Privacy,"yes") == 0)
+               if(Privacy)
                   sprintf(httplink,"<font size=%s color=%s><href=http://%s>%s",	\
                               FontSize,PrivacyStringColor,PrivacyString,PrivacyString);
                else
@@ -237,8 +244,8 @@ void gerarel(void)
                */
 
                sprintf(ltext110,"%s",text[110]);
-               for(s=ltext110; *s; ++s)
-                  *s=tolower(*s);
+               for(str=ltext110; *str; ++str)
+                  *str=tolower(*str);
 
                fprintf(fp_tt, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n<html>\n<head>\n  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
                css(fp_tt);
@@ -274,15 +281,13 @@ void gerarel(void)
             strcpy(html_old, html);
          } else bzero(ltext110, 50);
 
-         strcpy(wcrc,acccode);
-         if (getword(crc2,sizeof(crc2),wcrc,'/')<0) {
-            printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",tmp3);
-            exit(1);
-         }
+         strcpy(crc2,acccode);
+         str=strchr(crc2,'/');
+         if (str) *str='\0';
 
          if(strstr(crc2,"MISS") != 0)
-            oucache+=my_atoll(accbytes);
-         else incache+=my_atoll(accbytes);
+            oucache+=accbytes;
+         else incache+=accbytes;
 
          strcpy(oldurl,accurl);
 
@@ -293,7 +298,6 @@ void gerarel(void)
 
          strcpy(oldaccuser,accuser);
          strcpy(oldacccode,acccode);
-         strcpy(oldaccelap,accelap);
          strcpy(oldaccip,accip);
          strcpy(oldaccdia,accdia);
          strcpy(oldacchora,acchora);
@@ -313,8 +317,8 @@ void gerarel(void)
       strcpy(oldaccuser,accuser);
    gravatmpf(oldaccuser,wdirname,oldurl,nacc,nbytes,oldmsg,nelap,indexonly,incache,oucache);
    strcpy(wdirname,dirname);
-   gravager(wdirname,oldaccuser,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
-   strcpy(wdirname,dirname);
+   gravager(fp_gen,oldaccuser,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
+   fclose(fp_gen);
    day_totalize(tmp,oldaccuser,indexonly);
 
    tmpsort();
@@ -325,7 +329,7 @@ void gerarel(void)
    if(email[0] == '\0') {
       if(strstr(ReportType,"downloads") != 0) download_report();
 
-      if(strlen(DansGuardianConf) > 0) {
+      if(DansGuardianConf[0] != '\0') {
          strcpy(wdirname,dirname);
          dansguardian_log();
       }
@@ -346,7 +350,7 @@ void gerarel(void)
 
       if(smartfilter) smartfilter_report();
 
-      if(strlen(DansGuardianConf) > 0) dansguardian_report();
+      if(DansGuardianConf[0] != '\0') dansguardian_report();
 
       squidguard_report();
 
@@ -354,12 +358,12 @@ void gerarel(void)
 
       make_index();
 
-      if(strncmp(SuccessfulMsg,"yes",3) == 0) fprintf(stderr, "SARG: %s %s\n",text[47],dirname);
+      if(SuccessfulMsg) fprintf(stderr, "SARG: %s %s\n",text[47],dirname);
     } else {
       strcpy(wdirname,dirname);
       geramail(wdirname, debug, outdir, userip, email, TempDir);
 
-      if((strcmp(email,"stdout") != 0) && (strncmp(SuccessfulMsg,"yes",3) == 0))
+      if((strcmp(email,"stdout") != 0) && SuccessfulMsg)
             fprintf(stderr, "SARG: %s %s\n",text[48],email);
    }
 
@@ -431,7 +435,7 @@ static void maketmp_hour(const char *user, const char *dirname, int indexonly)
 }
 
 
-void gravatmp(const char *oldaccuser, const char *dirname, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, int indexonly, long long int incache, long long int oucache)
+void gravatmp(const char *oldaccuser, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, int indexonly, long long int incache, long long int oucache)
 {
 
    FILE *fp_ou;
@@ -440,28 +444,27 @@ void gravatmp(const char *oldaccuser, const char *dirname, const char *oldurl, l
    char val3[16];
    char val4[16];
    char val5[16];
-
    char wdirname[MAXLEN];
 
    if(indexonly) return;
    if(strstr(ReportType,"users_sites") == 0) return;
 
-   strcpy(wdirname,tmp);
-   strcat(wdirname,"/");
-   strcat(wdirname,oldaccuser);
-   strcat(wdirname,".utmp");
+   if (snprintf(wdirname,sizeof(wdirname),"%s/%s.utmp",tmp,oldaccuser)>=sizeof(wdirname)) {
+      fprintf(stderr,"SARG: Path too long %s/%s.utmp\n",tmp,oldaccuser);
+      exit(1);
+   }
 
    if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report) %s: %s\n",text[45],wdirname);
       exit(1);
    }
 
-   my_lltoa(nacc,val1,15);
-   my_lltoa(nbytes,val2,15);
-   my_lltoa(nelap,val3,15);
-   my_lltoa(incache,val4,15);
-   my_lltoa(oucache,val5,15);
-   fprintf(fp_ou,"%s %s %s %s %s %s %s\n",val1,val2,oldurl,oldmsg,val3,val4,val5);
+   my_lltoa(nacc,val1,0);
+   my_lltoa(nbytes,val2,0);
+   my_lltoa(nelap,val3,0);
+   my_lltoa(incache,val4,0);
+   my_lltoa(oucache,val5,0);
+   fprintf(fp_ou,"%s\t%s\t%s\t%s\t%s\t%s\t%s\n",val1,val2,oldurl,oldmsg,val3,val4,val5);
 
    fclose(fp_ou);
    ttopen=0;
@@ -477,7 +480,7 @@ void gravatmp(const char *oldaccuser, const char *dirname, const char *oldurl, l
 }
 
 
-static void gravatmp_hora(const char *dirname, const char *user, const char *data, const char *hora, const char *elap, const char *bytes, int indexonly)
+static void gravatmp_hora(const char *dirname, const char *user, const char *data, const char *hora, long long int elap, long long int bytes, int indexonly)
 {
 
    FILE *fp_ou;
@@ -485,18 +488,18 @@ static void gravatmp_hora(const char *dirname, const char *user, const char *dat
 
    if(indexonly || (strstr(ReportType,"users_sites") == 0)) return;
 
-   strcpy(wdirname,tmp);
-   strcat(wdirname,"/");
-   strcat(wdirname,user);
-   strcat(wdirname,".htmp");
+   if (snprintf(wdirname,sizeof(wdirname),"%s/%s.htmp",tmp,user)>=sizeof(wdirname)) {
+      fprintf(stderr,"SARG: Path too long %s/%s.htmp\n",tmp,user);
+      exit(1);
+   }
 
    if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report-2) %s: %s - %s\n",text[45],wdirname,strerror(errno));
       exit(1);
    }
 
-   if(strcmp(datetimeby,"bytes") == 0) fprintf(fp_ou,"%s %s %s\n",data,hora,bytes);
-   else fprintf(fp_ou,"%s %s %s\n",data,hora,elap);
+   if(strcmp(datetimeby,"bytes") == 0) fprintf(fp_ou,"%s\t%s\t%lld\n",data,hora,bytes);
+   else fprintf(fp_ou,"%s\t%s\t%lld\n",data,hora,elap);
 
    fclose(fp_ou);
 
@@ -504,26 +507,25 @@ static void gravatmp_hora(const char *dirname, const char *user, const char *dat
 }
 
 
-static void gravaporuser(const char *user, const char *dirname, const char *url, const char *ip, const char *data, const char *hora, const char *tam, const char *elap, int indexonly)
+static void gravaporuser(const char *user, const char *dirname, const char *url, const char *ip, const char *data, const char *hora, long long int tam, long long int elap, int indexonly)
 {
 
    FILE *fp_ou;
-
    char wdirname[MAXLEN];
 
    if(indexonly || (strstr(ReportType,"users_sites") == 0)) return;
 
-   strcpy(wdirname,tmp);
-   strcat(wdirname,"/");
-   strcat(wdirname,user);
-   strcat(wdirname,".ip");
+   if (snprintf(wdirname,sizeof(wdirname),"%s/%s.ip",tmp,user)>=sizeof(wdirname)) {
+      fprintf(stderr,"SARG: Path too long %s/%s.ip\n",tmp,user);
+      exit(1);
+   }
 
    if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report) %s: %s\n",text[45],wdirname);
       exit(1);
    }
 
-   fprintf(fp_ou,"%s %s %s %s %s %s\n",ip,url,data,hora,tam,elap);
+   fprintf(fp_ou,"%s\t%s\t%s\t%s\t%lld\t%lld\n",ip,url,data,hora,tam,elap);
 
    fclose(fp_ou);
 
@@ -536,27 +538,26 @@ static void gravatmpf(const char *oldaccuser, const char *dirname, const char *o
 {
 
    FILE *fp_ou;
-
    char wdirname[MAXLEN];
 
    if(indexonly || (strstr(ReportType,"users_sites") == 0)) return;
 
-   strcpy(wdirname,tmp);
-   strcat(wdirname,"/");
-   strcat(wdirname,oldaccuser);
-   strcat(wdirname,".utmp");
+   if (snprintf(wdirname,sizeof(wdirname),"%s/%s.utmp",tmp,oldaccuser)>=sizeof(wdirname)) {
+      fprintf(stderr,"SARG: Path too long %s/%s.utmp\n",tmp,oldaccuser);
+      exit(1);
+   }
 
    if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
       fprintf(stderr, "SARG: (report) %s: %s\n",text[45],wdirname);
       exit(1);
    }
 
-   my_lltoa(nacc,val1,15);
-   my_lltoa(nbytes,val2,15);
-   my_lltoa(nelap,val3,15);
-   my_lltoa(incache,val4,15);
-   my_lltoa(oucache,val5,15);
-   fprintf(fp_ou,"%s %s %s %s %s %s %s\n",val1,val2,oldurl,oldmsg,val3,val4,val5);
+   my_lltoa(nacc,val1,0);
+   my_lltoa(nbytes,val2,0);
+   my_lltoa(nelap,val3,0);
+   my_lltoa(incache,val4,0);
+   my_lltoa(oucache,val5,0);
+   fprintf(fp_ou,"%s\t%s\t%s\t%s\t%s\t%s\t%s\n",val1,val2,oldurl,oldmsg,val3,val4,val5);
 
    fclose(fp_ou);
    ttopen=0;
@@ -573,27 +574,14 @@ static void gravatmpf(const char *oldaccuser, const char *dirname, const char *o
 }
 
 
-static void gravager(char *dirname, const char *user, long long int nacc, const char *url, long long int nbytes, const char *ip, const char *hora, const char *dia, long long int nelap, long long int incache, long long int oucache)
+static void gravager(FILE *fp_gen, const char *user, long long int nacc, const char *url, long long int nbytes, const char *ip, const char *hora, const char *dia, long long int nelap, long long int incache, long long int oucache)
 {
-
-   FILE *fp_ou;
-
-   strcat(dirname,"/");
-   strcat(dirname,"sarg-general");
-
-   if((fp_ou=MY_FOPEN(dirname,"a"))==NULL){
-      fprintf(stderr, "SARG: (report) %s: %s\n",text[45],dirname);
-      exit(1);
-   }
-
-   my_lltoa(nacc,val1,15);
-   my_lltoa(nbytes,val2,15);
-   my_lltoa(nelap,val3,15);
-   my_lltoa(incache,val4,15);
-   my_lltoa(oucache,val5,15);
-   fprintf(fp_ou,"%s %s %s %s %s %s %s %s %s %s\n",user,val1,val2,url,ip,hora,dia,val3,val4,val5);
-
-   fclose(fp_ou);
+   my_lltoa(nacc,val1,0);
+   my_lltoa(nbytes,val2,0);
+   my_lltoa(nelap,val3,0);
+   my_lltoa(incache,val4,0);
+   my_lltoa(oucache,val5,0);
+   fprintf(fp_gen,"%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",user,val1,val2,url,ip,hora,dia,val3,val4,val5);
    return;
 
 }
@@ -602,7 +590,6 @@ static void grava_SmartFilter(const char *dirname, const char *user, const char
 {
 
    FILE *fp_ou;
-
    char wdirname[MAXLEN];
 
    sprintf(wdirname,"%s/smartfilter.unsort",dirname);
@@ -612,7 +599,7 @@ static void grava_SmartFilter(const char *dirname, const char *user, const char
       exit(1);
    }
 
-   fprintf(fp_ou,"%s %s %s %s %s %s\n",user,data,hora,ip,url,smart);
+   fprintf(fp_ou,"%s\t%s\t%s\t%s\t%s\t%s\n",user,data,hora,ip,url,smart);
    fputs("</body>\n</html>\n",fp_tt);
 
    fclose(fp_ou);
