@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,7 +31,7 @@ void smartfilter_report(void)
 {
 
    FILE *fp_in = NULL, *fp_ou = NULL, *fp_user = NULL;
-      
+
    char url[MAXLEN];
    char csort[255];
    char smart_in[MAXLEN];
@@ -51,6 +52,7 @@ void smartfilter_report(void)
    char *str;
    int  fuser=0;
    int cstatus;
+   struct getwordstruct gwarea;
 
    ouser[0]='\0';
 
@@ -115,9 +117,10 @@ void smartfilter_report(void)
    fprintf(fp_ou,"<tr><th bgcolor=%s><font size=\"%s\">%s</font></th><th bgcolor=\"%s\"><font size=\"%s\">%s</font></th><th bgcolor=\"%s\"><font size=\"%s\">%s</font></th><th bgcolor=\"%s\"><font size=\"%s\">%s</font></th><th bgcolor=\"%s\"><font size=\"%s\">%s</font></th></tr>\n",HeaderBgColor,FontSize,text[98],HeaderBgColor,FontSize,text[111],HeaderBgColor,FontSize,text[110],HeaderBgColor,FontSize,text[91],HeaderBgColor,FontSize,smartheader);
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if (getword(user,sizeof(user),buf,' ')<0 || getword(data,sizeof(data),buf,' ')<0 ||
-          getword(hora,sizeof(hora),buf,' ')<0 || getword(ip,sizeof(ip),buf,' ')<0 ||
-          getword(url,sizeof(url),buf,' ')<0 || getword(smartcat,sizeof(smartcat),buf,'\n')<0) {
+      getword_start(&gwarea,buf);
+      if (getword(user,sizeof(user),&gwarea,'\t')<0 || getword(data,sizeof(data),&gwarea,'\t')<0 ||
+          getword(hora,sizeof(hora),&gwarea,'\t')<0 || getword(ip,sizeof(ip),&gwarea,'\t')<0 ||
+          getword(url,sizeof(url),&gwarea,'\t')<0 || getword(smartcat,sizeof(smartcat),&gwarea,'\n')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",smart_ou);
          exit(1);
       }
@@ -133,8 +136,8 @@ void smartfilter_report(void)
          if(fuser) {
             fuser=0;
             fputs("</table>\n",fp_user);
-            if(strcmp(ShowSargInfo,"yes") == 0) {
-               zdate(ftime, DateFormat);
+            if(ShowSargInfo) {
+               zdate(ftime, sizeof(ftime), DateFormat);
                fprintf(fp_user,"<br><br><div align=\"center\"><font size=\"-2\">%s <a href=\"%s\">%s-%s</a> %s %s</font></div>\n",text[108],URL,PGM,VERSION,text[109],ftime);
             }
             fputs("</body>\n</html>\n",fp_user);
@@ -153,7 +156,7 @@ void smartfilter_report(void)
          fprintf(fp_user,"  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=%s\">\n",CharSet);
          fputs("</head>\n",fp_user);
 
-         if(strlen(FontFace) > 0) {
+         if(FontFace[0] != 0) {
             /*
             Before merging the sprintf and the fputs, the code looked like this:
             sprintf(html2,"<font face=%s>\n",FontFace);
@@ -185,8 +188,8 @@ void smartfilter_report(void)
 
    fputs("</table>\n",fp_ou);
 
-   if(strcmp(ShowSargInfo,"yes") == 0) {
-      zdate(ftime, DateFormat);
+   if(ShowSargInfo) {
+      zdate(ftime, sizeof(ftime), DateFormat);
       fprintf(fp_ou,"<br><br><div align=\"center\"><font size=\"-2\">%s <a href=\"%s\">%s-%s</a> %s %s</font></div>\n",text[108],URL,PGM,VERSION,text[109],ftime);
    }
 
@@ -195,8 +198,8 @@ void smartfilter_report(void)
    fclose(fp_ou);
    if(fp_user) {
       fputs("</table>\n",fp_user);
-      if(strcmp(ShowSargInfo,"yes") == 0) {
-         zdate(ftime, DateFormat);
+      if(ShowSargInfo) {
+         zdate(ftime, sizeof(ftime), DateFormat);
          fprintf(fp_user,"<br><br><div align=\"center\"><font size=\"-2\">%s <a href=\"%s\">%s-%s</a> %s %s</font></div>\n",text[108],URL,PGM,VERSION,text[109],ftime);
       }
       fputs("</body>\n</html>\n",fp_user);
