@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,11 +34,11 @@ void splitlog(const char *arq, char *df, int dfrom, int duntil, char *convert)
    char buf[MAXLEN];
    char data[30];
    char dia[11];
-   char hora[9];
    char wdata[20];
    time_t tt;
    int idata=0;
    struct tm *t;
+   struct getwordstruct gwarea;
 
    if(arq[0] == '\0')
       arq="/var/log/squid/access.log";
@@ -48,7 +49,8 @@ void splitlog(const char *arq, char *df, int dfrom, int duntil, char *convert)
    }
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if (getword(data,sizeof(data),buf,' ')<0) {
+      getword_start(&gwarea,buf);
+      if (getword(data,sizeof(data),&gwarea,' ')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",arq);
          exit(1);
       }
@@ -56,24 +58,23 @@ void splitlog(const char *arq, char *df, int dfrom, int duntil, char *convert)
       t=localtime(&tt);
 
       if(dfrom) {
-         strftime(wdata, 127, "%Y%m%d", t);
+         strftime(wdata, sizeof(wdata), "%Y%m%d", t);
          idata=atoi(wdata);
-	 if(idata < dfrom || idata > duntil)
-         continue;
+         if(idata < dfrom || idata > duntil)
+            continue;
       }
 
       if(strcmp(convert,"onvert") != 0) {
-         printf("%s %s",data,buf);
+         printf("%s %s",data,gwarea.current);
          continue;
       }
 
       if(strncmp(df,"e",1) == 0)
-         strftime(dia, 127, "%d/%m/%Y", t);
+         strftime(dia, sizeof(dia), "%d/%m/%Y", t);
        else
-         strftime(dia, 127, "%m/%d/%Y", t);
+         strftime(dia, sizeof(dia), "%m/%d/%Y", t);
 
-      sprintf(hora,"%02d:%02d:%02d",t->tm_hour,t->tm_min,t->tm_sec);
-      printf("%s %s %s",dia,hora,buf);
+      printf("%s %02d:%02d:%02d %s",dia,t->tm_hour,t->tm_min,t->tm_sec,gwarea.current);
    }
 
    fclose(fp_in);
