@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,22 +34,25 @@ static int nfiles_done = 0;
 static void read_log(const char *wentp, FILE *fp_ou)
 {
    FILE *fp_in = NULL;
-   char bufbsf[255];
    char leks[5], sep[2], res[MAXLEN];
    char mon[10], hour[15];
    char list[MAXLEN];
    char wdata[127];
+   char url[MAXLEN];
+   char urly[MAXLEN];
    int  idata=0;
    int  i;
    char *str;
+   struct getwordstruct gwarea;
+   struct getwordstruct gwarea1;
 
    if(debug) {
-      strcpy(buf,text[7]);
-      if (getword(urly,sizeof(urly),buf,' ')<0 || getword(href,sizeof(href),buf,' ')<0) {
+      getword_start(&gwarea,text[7]);
+      if (getword(urly,sizeof(urly),&gwarea,' ')<0 || getword(href,sizeof(href),&gwarea,' ')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s string.\n",text[7]);
          exit(1);
       }
-      debuga("%s squidGuard %s: %s",urly,buf,wentp);
+      debuga("%s squidGuard %s: %s",urly,gwarea.current,wentp);
    }
 
    /* With squidGuard, you can log groups in only one log file.
@@ -90,20 +94,21 @@ static void read_log(const char *wentp, FILE *fp_ou)
    }
 
    while (fgets(buf,sizeof(buf),fp_in) != NULL) {
-      if(strlen(SquidGuardLogFormat) > 0) {
-         strcpy(bufbsf,SquidGuardLogFormat);
+      getword_start(&gwarea,buf);
+      if(SquidGuardLogFormat[0] != '\0') {
+         getword_start(&gwarea1,SquidGuardLogFormat);
          leks[0]='\0';
-         if (getword(leks,sizeof(leks),bufbsf,'#')<0) {
+         if (getword(leks,sizeof(leks),&gwarea1,'#')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wentp);
             exit(1);
          }
          while(strcmp(leks,"end") != 0) {
-            if (getword(leks,sizeof(leks),bufbsf,'#')<0 || getword(sep,sizeof(sep),bufbsf,'#')<0) {
+            if (getword(leks,sizeof(leks),&gwarea1,'#')<0 || getword(sep,sizeof(sep),&gwarea1,'#')<0) {
                printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wentp);
                exit(1);
             }
             if(strcmp(leks,"end") != 0) {
-               if (getword(res,sizeof(res),buf,sep[0])<0) {
+               if (getword(res,sizeof(res),&gwarea,sep[0])<0) {
                   printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wentp);
                   exit(1);
                }
@@ -128,34 +133,47 @@ static void read_log(const char *wentp, FILE *fp_ou)
             }
          }
       } else {
-         if (getword(year,sizeof(year),buf,'-')<0 || getword(mon,sizeof(mon),buf,'-')<0 ||
-             getword(day,sizeof(day),buf,' ')<0 || getword(hour,sizeof(hour),buf,' ')<0 ||
-             getword(list,sizeof(list),buf,'/')<0 || getword(list,sizeof(list),buf,'/')<0 ||
-             getword(tmp5,sizeof(tmp5),buf,'/')<0 || getword(tmp5,sizeof(tmp5),buf,'/')<0 ||
-             getword(url,sizeof(url),buf,' ')<0 ||
-             getword(ip,sizeof(ip),buf,'/')<0 || getword(user,sizeof(user),buf,' ')<0 ||
-             getword(user,sizeof(user),buf,' ')<0) {
+         if (getword(year,sizeof(year),&gwarea,'-')<0 || getword(mon,sizeof(mon),&gwarea,'-')<0 ||
+             getword(day,sizeof(day),&gwarea,' ')<0 || getword(hour,sizeof(hour),&gwarea,' ')<0 ||
+             getword_skip(MAXLEN,&gwarea,'/')<0 || getword(list,sizeof(list),&gwarea,'/')<0 ||
+             getword_skip(MAXLEN,&gwarea,' ')<0 || getword(url,sizeof(url),&gwarea,' ')<0 ||
+             getword(ip,sizeof(ip),&gwarea,'/')<0 || getword_skip(MAXLEN,&gwarea,' ')<0 ||
+             getword(user,sizeof(user),&gwarea,' ')<0) {
             printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",wentp);
             exit(1);
           }
+          /*
+          The URL may be "http://url:port/data" if the method is GET or simply "url:port/" if the method is CONNECT.
+          The following code removes the protocol:// if it is detected and always truncates the URL after the domain name.
+          It will fail if the URL doesn't start with the protocol and contains two consecutive / in the path (i.e.
+          the URL is not normalized).
+          */
           str=strchr(url,'/');
-          if (str) *str='\0';
+          if (str) {
+             if (str[1]=='/') {
+               str+=2;
+               for (i=0 ; *str && *str!='/' ; i++) url[i]=*str++;
+               url[i]='\0';
+             } else {
+               *str='\0';
+             }
+          }
       }
 
       sprintf(warea,"%s%s%s",year,mon,day);
       sprintf(wdata,"%s%s%s",year,mon,day);
       idata = atoi(wdata);
 
-      if(strcmp(SquidguardIgnoreDate,"on") == 0) {
+      if(SquidguardIgnoreDate) {
          if(idata < dfrom || idata > duntil)
             continue;
       }
 
       if (strcmp(user,"-") == 0) {
          strcpy(user,ip);
-         memset(ip,0,sizeof(ip));
+         ip[0]='\0';
       }
-      fprintf(fp_ou,"%s %s%s%s %s %s %s %s\n",user,year,mon,day,hour,ip,url,list);
+      fprintf(fp_ou,"%s\t%s%s%s\t%s\t%s\t%s\t%s\n",user,year,mon,day,hour,ip,url,list);
       squidguard_count++;
    }
    if (fp_in) fclose(fp_in);
@@ -181,7 +199,7 @@ void squidguard_log(void)
    if(strlen(SquidGuardConf) < 1 && strlen(SquidGuardLogAlternate) < 1)
      return;
 
-   if (strlen(SquidGuardLogAlternate) > 0)
+   if (SquidGuardLogAlternate[0] != '\0')
       SquidGuardConf[0]='\0';
 
    sprintf(guard_in,"%s/squidguard.unsort",tmp);
@@ -195,7 +213,7 @@ void squidguard_log(void)
    bzero(mon, 4);
    bzero(year, 5);
 
-   if(strcmp(SquidguardIgnoreDate,"on") == 0) {
+   if(SquidguardIgnoreDate) {
       if(strcmp(df,"e") == 0) {
          strncpy(day,period,2);
          strncpy(mon,period+2,3);
@@ -225,7 +243,7 @@ void squidguard_log(void)
       }
    }
 
-   if(strlen(SquidGuardConf) > 0) {
+   if(SquidGuardConf[0] != 0) {
       if(access(SquidGuardConf, R_OK) != 0) {
          debuga("Cannot open squidGuard config file: %s",SquidGuardConf);
          exit(1);
@@ -238,12 +256,19 @@ void squidguard_log(void)
 
       logdir[0]=0;
       while(fgets(buf,sizeof(buf),fp_guard)!=NULL) {
-         if(strstr(buf,"\n") != 0)
-            buf[strlen(buf)-1]='\0';
+         fixendofline(buf);
          if((str=get_param_value("logdir",buf))!=NULL) {
-            if (getword(logdir,sizeof(logdir),str,' ')<0) {
-               printf("SARG: Maybe you have an invalid logdir line in your %s file.\n",SquidGuardConf);
-               exit(1);
+            /*
+            We want to tolerate spaces inside the directory name but we must also
+            remove the trailing spaces left by the editor after the directory name.
+            This should not be a problem as nobody use a file name with trailing spaces.
+            */
+            for (y=strlen(str)-1 ; y>=0 && (unsigned char)str[y]<=' ' ; y--);
+            if (y>=sizeof(logdir)-1) y=sizeof(logdir)-2;
+            logdir[y+1] = '\0';
+            while (y>=0) {
+               logdir[y] = str[y];
+               y--;
             }
          } else if((str=get_param_value("log",buf))!=NULL) {
             if((str2=get_param_value("anonymous",str))!=NULL)
