@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,9 +34,6 @@ void topsites(void)
 
    char url[MAXLEN];
    char ourl[MAXLEN];
-   char nacc[20];
-   char nbytes[20];
-   char ntime[20];
    char ntemp[255];
    char ttnacc[20];
    char ttnbytes[20];
@@ -50,14 +48,18 @@ void topsites(void)
    char period[100];
    char sortf[10];
    char sortt[10];
+   long long int nacc;
+   long long int nbytes;
+   long long int ntime;
    long long int tnacc=0;
    long long int tnbytes=0;
    long long int tntime=0;
    long long int twork1=0, twork2=0, twork3=0;
    int regs=0;
    int cstatus;
+   struct getwordstruct gwarea;
 
-   if(strcmp(Privacy,"yes") == 0)
+   if(Privacy)
       return;
 
    sprintf(general,"%s/sarg-general",dirname);
@@ -102,50 +104,51 @@ void topsites(void)
    }
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if (getword(url,sizeof(url),buf,' ')<0) {
+      getword_start(&gwarea,buf);
+      if (getword(url,sizeof(url),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have an invalid user in your %s file of the topsites.\n",general2);
          exit(1);
       }
       if(strcmp(url,"TOTAL") == 0) {
-         if (getword(ttnacc,sizeof(ttnacc),buf,' ')<0) {
+         if (getword(ttnacc,sizeof(ttnacc),&gwarea,'\t')<0) {
             printf("SARG: Maybe you have an invalid total number of access in your %s file of the topsites.\n",general2);
             exit(1);
          }
-         if (getword(ttnbytes,sizeof(ttnbytes),buf,' ')<0) {
+         if (getword(ttnbytes,sizeof(ttnbytes),&gwarea,'\t')<0) {
             printf("SARG: Maybe you have an invalid total number of bytes in your %s file of the topsites.\n",general2);
             exit(1);
          }
-         if (getword(ttntime,sizeof(ttntime),buf,' ')<0) {
+         if (getword(ttntime,sizeof(ttntime),&gwarea,'\t')<0) {
             printf("SARG: Maybe you have an invalid total time in your %s file of the topsites.\n",general2);
             exit(1);
          }
          continue;
       }
-      if (getword(nacc,sizeof(nacc),buf,' ')<0) {
+      if (getword_atoll(&nacc,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have an invalid number of access in your %s file of the topsites.\n",general2);
          exit(1);
       }
-      if (getword(nbytes,sizeof(nbytes),buf,' ')<0) {
+      if (getword_atoll(&nbytes,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have an invalid number of bytes in your %s file of the topsites.\n",general2);
          exit(1);
       }
-      if (getword(url,sizeof(url),buf,' ')<0) {
+      if (getword(url,sizeof(url),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have an invalid url in your %s file of the topsites.\n",general2);
          exit(1);
       }
-      if (getword(ntemp,sizeof(ntemp),buf,' ')<0) {
+      if (getword(ntemp,sizeof(ntemp),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in column 5 in your %s file of the topsites.\n",general2);
          exit(1);
       }
-      if (getword(ntemp,sizeof(ntemp),buf,' ')<0) {
+      if (getword(ntemp,sizeof(ntemp),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in column 6 in your %s file of the topsites.\n",general2);
          exit(1);
       }
-      if (getword(ntemp,sizeof(ntemp),buf,' ')<0) {
+      if (getword(ntemp,sizeof(ntemp),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in column 7 in your %s file of the topsites.\n",general2);
          exit(1);
       }
-      if (getword(ntime,sizeof(ntime),buf,' ')<0) {
+      if (getword_atoll(&ntime,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in column 8 in your %s file of the topsites.\n",general2);
          exit(1);
       }
@@ -159,22 +162,22 @@ void topsites(void)
          my_lltoa(tnacc,val1,15);
          my_lltoa(tnbytes,val2,15);
          my_lltoa(tntime,val3,15);
-         fprintf(fp_ou,"%s %s %s %s\n",val1,val2,val3,ourl);
+         fprintf(fp_ou,"%s\t%s\t%s\t%s\n",val1,val2,val3,ourl);
          strcpy(ourl,url);
          tnacc=0;
          tnbytes=0;
          tntime=0;
       }
 
-      tnacc+=my_atoll(nacc);
-      tnbytes+=my_atoll(nbytes);
-      tntime+=my_atoll(ntime);
+      tnacc+=nacc;
+      tnbytes+=nbytes;
+      tntime+=ntime;
    }
 
    my_lltoa(tnacc,val1,15);
    my_lltoa(tnbytes,val2,15);
    my_lltoa(tntime,val3,15);
-   fprintf(fp_ou,"%s %s %s %s\n",val1,val2,val3,ourl);
+   fprintf(fp_ou,"%s\t%s\t%s\t%s\n",val1,val2,val3,ourl);
 
    fclose(fp_in);
    fclose(fp_ou);
@@ -230,7 +233,7 @@ void topsites(void)
    fprintf(fp_ou,"<tr><th class=\"title\">%s</th></tr>\n",Title);
 
    fprintf(fp_ou,"<tr><td class=\"header3\">%s: %s</td></tr>\n",text[89],period);
-   fprintf(fp_ou,"<tr><th class=\"header3\">%s %s %s</th></tr>\n",text[83],TopSitesNum,text[84]);
+   fprintf(fp_ou,"<tr><th class=\"header3\">%s %d %s</th></tr>\n",text[83],TopSitesNum,text[84]);
    fputs("</table></div>\n",fp_ou);
 
    fputs("<div align=\"center\"><table cellpadding=\"1\" cellspacing=\"2\">\n",fp_ou);
@@ -244,28 +247,29 @@ void topsites(void)
    ntopsites = 0;
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if(regs>atoi(TopSitesNum))
+      if(regs>TopSitesNum)
          break;
-      if (getword(nacc,sizeof(nacc),buf,' ')<0) {
+      getword_start(&gwarea,buf);
+      if (getword_atoll(&nacc,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",sites);
          exit(1);
       }
-      if (atoi(nacc) == 0) continue;
-      if (getword(nbytes,sizeof(nbytes),buf,' ')<0 ||
-          getword(ntime,sizeof(ntime),buf,' ')<0 || getword(url,sizeof(url),buf,' ')<0) {
+      if (nacc == 0) continue;
+      if (getword_atoll(&nbytes,&gwarea,'\t')<0 ||
+          getword_atoll(&ntime,&gwarea,'\t')<0 || getword(url,sizeof(url),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",sites);
          exit(1);
       }
 
-      twork1=my_atoll(nacc);
-      twork2=my_atoll(nbytes);
-      twork3=my_atoll(ntime);
+      twork1=nacc;
+      twork2=nbytes;
+      twork3=ntime;
 
       strcpy(wwork1,fixnum(twork1,1));
       strcpy(wwork2,fixnum(twork2,1));
       strcpy(wwork3,fixtime(twork3));
 
-      if(strlen(BlockIt) > 0)
+      if(BlockIt[0] != '\0')
          sprintf(BlockImage,"<a href=\"%s%s?url=%s\"><img src=\"../images/sarg-squidguard-block.png\" border=\"0\"></a>&nbsp;",wwwDocumentRoot,BlockIt,url);
       else BlockImage[0]='\0';
 
