@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -35,14 +36,17 @@ void topuser(void)
    long long int tnincache=0, tnoucache=0, ttnincache=0, ttnoucache=0;
    long long int twork;
    long long int twork2;
+   long long int nbytes;
+   long long int nacc;
+   long long int elap, incac, oucac;
    float perc=0.00;
    float perc2=0.00;
    float inperc=0.00, ouperc=0.00;
    int posicao=0;
    char olduser[MAXLEN], csort[MAXLEN], period[MAXLEN], arqper[MAXLEN];
    char wger[MAXLEN], top1[MAXLEN], top2[MAXLEN], top3[MAXLEN];
-   char user[MAXLEN], nacc[20], nbytes[20], preg[MAXLEN], tusr[MAXLEN];
-   char ip[MAXLEN], time[30], date[30], elap[30], incac[30], oucac[30];
+   char user[MAXLEN], preg[MAXLEN], tusr[MAXLEN];
+   char ip[MAXLEN], time[30], date[30];
    char ipantes[MAXLEN], nameantes[MAXLEN];
    char sfield[10]="2,2";
    char order[255]="-r";
@@ -60,6 +64,8 @@ void topuser(void)
    char href1end[5];
    char href2end[5];
    char href3end[5];
+   char url[MAXLEN];
+   struct getwordstruct gwarea;
 
    ipantes[0]='\0';
    nameantes[0]='\0';
@@ -90,46 +96,47 @@ void topuser(void)
    totuser=0;
 
    while(fgets(warea,sizeof(warea),fp_in)) {
-      if (getword(user,sizeof(user),warea,' ')<0) {
+      getword_start(&gwarea,warea);
+      if (getword(user,sizeof(user),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken user in your %s file.\n",wger);
          exit(1);
       }
       if(strcmp(user,"TOTAL") == 0) {
          continue;
       }
-      if (getword(nacc,sizeof(nacc),warea,' ')<0) {
+      if (getword_atoll(&nacc,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken number of access in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(nbytes,sizeof(nbytes),warea,' ')<0) {
+      if (getword_atoll(&nbytes,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken number of bytes in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(url,sizeof(url),warea,' ')<0) {
+      if (getword(url,sizeof(url),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken url in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(ip,sizeof(ip),warea,' ')<0) {
+      if (getword(ip,sizeof(ip),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken user's IP in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(time,sizeof(time),warea,' ')<0) {
+      if (getword(time,sizeof(time),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken time in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(date,sizeof(date),warea,' ')<0) {
+      if (getword(date,sizeof(date),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken date in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(elap,sizeof(elap),warea,' ')<0) {
+      if (getword_atoll(&elap,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken download duration in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(incac,sizeof(incac),warea,' ')<0) {
+      if (getword_atoll(&incac,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken in cache download in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(oucac,sizeof(oucac),warea,' ')<0) {
+      if (getword_atoll(&oucac,&gwarea,'\n')<0) {
          printf("SARG: Maybe you have a broken not in cache download in your %s file.\n",wger);
          exit(1);
       }
@@ -142,7 +149,7 @@ void topuser(void)
             my_lltoa(tnelap,val3,15);
             my_lltoa(tnincache,val4,15);
             my_lltoa(tnoucache,val5,15);
-            fprintf(fp_top2,"%s %s %s %s %s %s\n",olduser,val1,val2,val3,val4,val5);
+            fprintf(fp_top2,"%s\t%s\t%s\t%s\t%s\t%s\n",olduser,val1,val2,val3,val4,val5);
 
             ttnbytes+=tnbytes;
             ttnacc+=tnacc;
@@ -158,11 +165,11 @@ void topuser(void)
          tnoucache=0;
       }
 
-      tnbytes+=my_atoll(nbytes);
-      tnacc+=my_atoll(nacc);
-      tnelap+=my_atoll(elap);
-      tnincache+=my_atoll(incac);
-      tnoucache+=my_atoll(oucac);
+      tnbytes+=nbytes;
+      tnacc+=nacc;
+      tnelap+=elap;
+      tnincache+=incac;
+      tnoucache+=oucac;
    }
 
    if (olduser[0] != '\0') {
@@ -171,7 +178,7 @@ void topuser(void)
       my_lltoa(tnelap,val3,15);
       my_lltoa(tnincache,val4,15);
       my_lltoa(tnoucache,val5,15);
-      fprintf(fp_top2,"%s %s %s %s %s %s\n",olduser,val1,val2,val3,val4,val5);
+      fprintf(fp_top2,"%s\t%s\t%s\t%s\t%s\t%s\n",olduser,val1,val2,val3,val4,val5);
 
       ttnbytes+=tnbytes;
       ttnacc+=tnacc;
@@ -219,7 +226,7 @@ void topuser(void)
 
    unlink(top2);
 
-   fprintf(fp_top1,"TOTAL %s %s %s %s %s\n",val1,val2,val3,val4,val5);
+   fprintf(fp_top1,"TOTAL\t%s\t%s\t%s\t%s\t%s\n",val1,val2,val3,val4,val5);
    fclose(fp_top1);
 
    /*
@@ -273,15 +280,15 @@ void topuser(void)
 
    fputs("</table></div>\n",fp_top3);
    fputs("<div align=\"center\"><table cellpadding=\"1\" cellspacing=\"2\">\n",fp_top3);
-   fputs("<tr><td><br></td><td></td></tr>\n",fp_top3);
+   fputs("<tr><td>&nbsp;</td><td>&nbsp;</td></tr>\n",fp_top3);
 
-   if(strstr(ReportType,"topsites") != 0 && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"topsites.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[119]);
-   if(strstr(ReportType,"sites_users") != 0 && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"siteuser.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[85]);
+   if(strstr(ReportType,"topsites") != 0 && !Privacy) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"topsites.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[119]);
+   if(strstr(ReportType,"sites_users") != 0 && !Privacy) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"siteuser.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[85]);
    if(dansguardian_count) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"dansguardian.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[128]);
    if(squidguard_count) fprintf(fp_top3,"<tr><td class=\"link\" colspan=11><a href=\"squidguard.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[122]);
-   if (strstr(ReportType,"downloads") != 0 && download_count && strcmp(Privacy,"yes") != 0 && ndownload) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"download.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[125]);
-   if (strstr(ReportType,"denied") != 0 && denied_count && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"denied.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[118]);
-   if (strstr(ReportType,"auth_failures") != 0 && authfail_count && strcmp(Privacy,"yes") != 0) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"authfail.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[117]);
+   if (strstr(ReportType,"downloads") != 0 && download_count && !Privacy && ndownload) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"download.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[125]);
+   if (strstr(ReportType,"denied") != 0 && denied_count && !Privacy) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"denied.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[118]);
+   if (strstr(ReportType,"auth_failures") != 0 && authfail_count && !Privacy) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"authfail.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[117]);
    if(smartfilter) fprintf(fp_top3,"<tr><td class=\"link\" colspan=\"11\"><a href=\"smartfilter.html\"><font class=\"link\">%s</font></a><font class=\"text\"></font></td></tr>\n",text[116]);
    fputs("<tr><td></td></tr>\n",fp_top3);
 
@@ -366,56 +373,53 @@ void topuser(void)
    ntopuser = 0;
 
    while(fgets(warea,sizeof(warea),fp_top1)) {
-      if (getword(user,sizeof(user),warea,' ')<0) {
+      getword_start(&gwarea,warea);
+      if (getword(user,sizeof(user),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken user in your %s file.\n",top1);
          exit(1);
       }
-      if (getword(nbytes,sizeof(nbytes),warea,' ')<0) {
+      if (getword_atoll(&nbytes,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken number of bytes in your %s file.\n",top1);
          exit(1);
       }
-      if (getword(nacc,sizeof(nacc),warea,' ')<0) {
+      if (getword_atoll(&nacc,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken number of access in your %s file.\n",top1);
          exit(1);
       }
-      if (getword(elap,sizeof(elap),warea,' ')<0) {
+      if (getword_atoll(&elap,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken elpased time in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(incac,sizeof(incac),warea,' ')<0) {
+      if (getword_atoll(&incac,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken in-cache size in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(oucac,sizeof(oucac),warea,' ')<0) {
+      if (getword_atoll(&oucac,&gwarea,'\n')<0) {
          printf("SARG: Maybe you have a broken out-of-cache size in your %s file.\n",wger);
          exit(1);
       }
-      if(atoi(nacc) < 1) {
+      if(nacc < 1) {
          continue;
       }
       ntopuser = 1;
       if(TopUsersNum > 0 && topcount >= TopUsersNum) goto final;
       strcpy(user2,user);
-      tnbytes=my_atoll(nbytes);
+      tnbytes=nbytes;
 
-      if(tnbytes) {
-         perc=tnbytes * 100;
-         perc=perc / ttnbytes;
+      if(ttnbytes) {
+         perc=tnbytes * 100. / ttnbytes;
       } else perc = 0;
 
-      if(atol(elap)) {
-         perc2=atol(elap);
-         perc2=((perc2 * 100) / ttnelap);
+      if(ttnelap) {
+         perc2=elap * 100. / ttnelap;
       } else perc2 = 0;
 
-      if(atol(incac)) {
-         inperc=atol(incac);
-         inperc=((inperc * 100) / tnbytes);
+      if(tnbytes) {
+         inperc=incac * 100. / tnbytes;
       } else inperc = 0;
 
-      if(atol(oucac)) {
-         ouperc=atol(oucac);
-         ouperc=((ouperc * 100) / tnbytes);
+      if(tnbytes) {
+         ouperc=oucac * 100. / tnbytes;
       } else ouperc = 0;
 
       if(strcmp(user,"TOTAL") != 0){
@@ -435,16 +439,16 @@ void topuser(void)
          strcpy(href3end,"");
       }
 
-      if(strcmp(Graphs,"yes") != 0) {
+      if(!Graphs || GraphFont[0]=='\0') {
          bzero(href3,MAXLEN);
          strcpy(href3end,"");
       }
 
-      tnelap=my_atoll(elap);
+      tnelap=elap;
 
       if(userip) {
          fixip(user2);
-         if(strcmp(Ip2Name,"yes") == 0) {
+         if(Ip2Name) {
             if(strcmp(user2,ipantes) != 0) {
                strcpy(ipantes,user2);
                ip2name(user2,sizeof(user2));
@@ -468,7 +472,7 @@ void topuser(void)
       if(strcmp(user2,"TOTAL") != 0) {
          get_usertab_name(user2,name,sizeof(name));
 
-         if((strcmp(Ip2Name,"yes") == 0) &&
+         if(Ip2Name &&
             ((str=(char *) strstr(name, ".")) != (char *) NULL) &&
             ((str=(char *) strstr(str+1, ".")) != (char *) NULL))
             ip2name(name,sizeof(name));
@@ -476,9 +480,7 @@ void topuser(void)
          name[0]='\0';
       }
 
-      twork=my_atoll(nacc);
-      my_lltoa(twork,nacc,0);
-      strcpy(wwork1,fixnum(twork,1));
+      strcpy(wwork1,fixnum(nacc,1));
       strcpy(wwork2,fixnum(tnbytes,1));
       strcpy(wwork3,fixnum2(tnelap,1));
 
@@ -557,18 +559,16 @@ void topuser(void)
          subs(name,sizeof(name),"_",".");
       }
 
-      sprintf(preg,"<tr><td %s>%s</td><td %s>%s</td><td %s>%s%s%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s<td %s>%s</td></tr>\n",hbc1,val1,hbc2,val2,hbc2,href1,name,href1end,hbc3,wwork1,hbc4,wwork2,hbc5,val3,hbc6,val4,hbc7,val5,hbc8,val6,hbc9,wwork3,hbc10,val7);
+      sprintf(preg,"<tr><td %s>%s</td><td %s>%s</td><td %s>%s%s%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td><td %s>%s</td></tr>\n",hbc1,val1,hbc2,val2,hbc2,href1,name,href1end,hbc3,wwork1,hbc4,wwork2,hbc5,val3,hbc6,val4,hbc7,val5,hbc8,val6,hbc9,wwork3,hbc10,val7);
 
       if(strstr(user,"TOTAL") != 0) {
 
-         if(atol(incac)) {
-            inperc=ttnbytes / 100;
-            inperc=atol(incac) / inperc;
+         if(ttnbytes) {
+            inperc=incac * 100. / ttnbytes;
          } else inperc = 0;
 
-         if(atol(oucac)) {
-            ouperc=ttnbytes / 100;
-            ouperc=atol(oucac) / ouperc;
+         if(ttnbytes) {
+            ouperc=oucac * 100. / ttnbytes;
          } else ouperc = 0;
 
          sprintf(wwork1,"%s",fixnum(ttnacc,1));
@@ -629,8 +629,13 @@ void topuser(void)
    if(ttnbytes) tnbytes=ttnbytes / totuser;
    else tnbytes=0;
 
-   twork=ttnacc/totuser;
-   twork2=ttnelap/totuser;
+   if (totuser>0) {
+      twork=ttnacc/totuser;
+      twork2=ttnelap/totuser;
+   } else {
+      twork=0;
+      twork2=0;
+   }
    strcpy(wwork1,fixnum(twork,1));
    strcpy(wwork2,fixnum(tnbytes,1));
    strcpy(wwork3,fixnum2(twork2,1));
