@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,7 +31,7 @@ void day_totalize(const char *tmp, const char *user, int indexonly)
 {
 
    FILE *fp_in, *fp_ou;
-      
+
    char data[20];
    char hora[20];
    char min[20];
@@ -38,16 +39,14 @@ void day_totalize(const char *tmp, const char *user, int indexonly)
    char odata[20];
    char ohora[20];
    char oelap[20];
-   char hm[20];
-   char ohm[20];
    char csort[255];
    char wdirname[MAXLEN];
    char sortout[MAXLEN];
    char arqout[MAXLEN];
    int  regs=0;
    long long int telap=0;
-   long long int tused=0;
    int cstatus;
+   struct getwordstruct gwarea;
 
    if(indexonly) return;
    if(strstr(ReportType,"users_sites") == 0) return;
@@ -77,52 +76,35 @@ void day_totalize(const char *tmp, const char *user, int indexonly)
    }
 
    while(fgets(buf,sizeof(buf),fp_in)!=NULL) {
-      if(strstr(buf,"\n") != 0)
-         buf[strlen(buf)-1]='\0';
-         
-      if (getword(data,sizeof(data),buf,' ')<0 || getword(hora,sizeof(hora),buf,':')<0 ||
-          getword(min,sizeof(min),buf,':')<0 || getword(elap,sizeof(elap),buf,' ')<0 ||
-          getword(elap,sizeof(elap),buf,0)<0) {
+      fixendofline(buf);
+      getword_start(&gwarea,buf);
+      if (getword(data,sizeof(data),&gwarea,'\t')<0 || getword(hora,sizeof(hora),&gwarea,':')<0 ||
+          getword(min,sizeof(min),&gwarea,':')<0 || getword_skip(20,&gwarea,'\t')<0 ||
+          getword(elap,sizeof(elap),&gwarea,0)<0) {
          printf("SARG: Maybe you have a broken record or garbage in your %s file.\n",sortout);
          exit(1);
       }
-      sprintf(hm,"%s%s",hora,min);
 
       if(!regs) {
          strcpy(odata,data);
          strcpy(ohora,hora);
          strcpy(oelap,elap);
-         strcpy(ohm,hm);
          regs++;
       }
 
       if(strcmp(hora,ohora) != 0 || strcmp(data,odata) != 0) {
-         if(tused > telap)
-            tused=telap;
-
          my_lltoa(telap,val1,15);
-         fprintf(fp_ou,"%s %s %s\n",odata,ohora,val1);
+         fprintf(fp_ou,"%s\t%s\t%s\n",odata,ohora,val1);
          strcpy(odata,data);
          strcpy(ohora,hora);
-         strcpy(ohm,hm);
          telap=0;
-         tused=0;
-      }
-
-      if(strcmp(ohm,hm) != 0) {
-         tused+=60000;
-         strcpy(ohm,hm);
       }
 
       telap+=my_atoll(elap);
-          
    }
 
-   if(tused > telap)
-      tused=telap;
-
    my_lltoa(telap,val1,15);
-   fprintf(fp_ou,"%s %s %s\n",data,hora,val1);
+   fprintf(fp_ou,"%s\t%s\t%s\n",data,hora,val1);
 
    fclose(fp_in);
    fclose(fp_ou);
