@@ -1,10 +1,11 @@
 /*
- * AUTHOR: Pedro Lineu Orso                         pedro.orso@gmail.com
- *                                                            1998, 2009
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
+ *                                                            1998, 2010
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
+ * Support:
+ *     http://sourceforge.net/projects/sarg/forums/forum/363374
  * ---------------------------------------------------------------------
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -33,10 +34,13 @@ int totalger(const char *dirname, int debug, const char *outdir)
    long long int tnbytes=0;
    long long int telap=0;
    long long int tincache=0, toucache=0;
-   char wger[MAXLEN], user[MAXLEN], nacc[16], nbytes[16], url[MAXLEN];
-   char ip[MAXLEN], hora[9], data[15], elap[16];
-   char incac[30], oucac[30];
+   long long int nacc, nbytes;
+   long long int elap;
+   long long int incac, oucac;
+   char wger[MAXLEN], user[MAXLEN], url[MAXLEN];
+   char ip[MAXLEN], hora[9], data[15];
    char warea[MAXLEN];
+   struct getwordstruct gwarea;
 
    strcpy(wger,dirname);
    strcat(wger,"/sarg-general");
@@ -49,51 +53,52 @@ int totalger(const char *dirname, int debug, const char *outdir)
    while(fgets(warea,sizeof(warea),fp_in))
    {
       //printf("%s\n",warea);
-      if (getword(user,sizeof(user),warea,' ')<0) {
+      getword_start(&gwarea,warea);
+      if (getword(user,sizeof(user),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken user in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(nacc,sizeof(nacc),warea,' ')<0) {
+      if (getword_atoll(&nacc,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken number of access in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(nbytes,sizeof(nbytes),warea,' ')<0) {
+      if (getword_atoll(&nbytes,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken number of bytes in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(url,sizeof(url),warea,' ')<0) {
+      if (getword(url,sizeof(url),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken url in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(ip,sizeof(ip),warea,' ')<0) {
+      if (getword(ip,sizeof(ip),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken source IP address in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(hora,sizeof(hora),warea,' ')<0) {
+      if (getword(hora,sizeof(hora),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken time in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(data,sizeof(data),warea,' ')<0) {
+      if (getword(data,sizeof(data),&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken date in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(elap,sizeof(elap),warea,' ')<0) {
+      if (getword_atoll(&elap,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken elapsed time in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(incac,sizeof(incac),warea,' ')<0) {
+      if (getword_atoll(&incac,&gwarea,'\t')<0) {
          printf("SARG: Maybe you have a broken in cache column in your %s file.\n",wger);
          exit(1);
       }
-      if (getword(oucac,sizeof(oucac),warea,0)<0) {
+      if (getword_atoll(&oucac,&gwarea,'\n')<0) {
          printf("SARG: Maybe you have a broken not in cache column in your %s file.\n",wger);
          exit(1);
       }
-      tnacc+=my_atoll(nacc);
-      tnbytes+=my_atoll(nbytes);
-      telap+=my_atoll(elap);
-      tincache+=my_atoll(incac);
-      toucache+=my_atoll(oucac);
+      tnacc+=nacc;
+      tnbytes+=nbytes;
+      telap+=elap;
+      tincache+=incac;
+      toucache+=oucac;
    }
 
    fclose(fp_in);
@@ -113,7 +118,7 @@ int totalger(const char *dirname, int debug, const char *outdir)
    my_lltoa(telap,val3,15);
    my_lltoa(tincache,val4,15);
    my_lltoa(toucache,val5,15);
-   fprintf(fp_ou,"TOTAL %s %s %s %s %s\n",val1,val2,val3,val4,val5);
+   fprintf(fp_ou,"TOTAL\t%s\t%s\t%s\t%s\t%s\n",val1,val2,val3,val4,val5);
    fclose(fp_ou);
 
    return (0);
